package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.AlarmTemplate;
import com.bocloud.cms.intf.AlarmTemplateService;
import com.bocloud.cms.model.DeleteIdList;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 告警模板
 *
 * @author fanzx
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@RestController
@Tag(name = "告警模板")
@RequestMapping("/v1/templates")
public class AlarmTemplateController {

    @Autowired
    private AlarmTemplateService alarmTemplateService;


    /**
     * 告警模板新增
     *
     * @param alarmTemplate 参数模板参数
     * @param context       用户上下文
     * @return 是否成功创建
     */
    @PostMapping
    @Operation(summary = "新增告警模板")
    public GeneralResult create(@RequestBody AlarmTemplate alarmTemplate,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.create(alarmTemplate, context);
    }


    /**
     * 告警模板修改
     *
     * @param alarmTemplate 模板参数修改
     * @param context       用户上下文
     * @return 是否修改成功
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改告警模板")
    public GeneralResult modify(@RequestBody AlarmTemplate alarmTemplate,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.modify(alarmTemplate, context);
    }


    /**
     * 告警模板删除
     *
     * @param id      模板id
     * @param context 用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除告警模板")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.remove(id, context);
    }

    /**
     * 获取告警模板列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 数据结果集
     */
    @GetMapping
    @Operation(summary = "告警模板列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.list(pager, context);
    }


    /**
     * 告警模板详情
     *
     * @param id 告警模板id
     * @return 告警模板的实体类详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "告警模板详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return alarmTemplateService.detail(id);
    }


    /**
     * 批量删除告警模板
     *
     * @param param   过滤参数
     * @param context 用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping
    @Operation(summary = "批量删除告警模板")
    public GeneralResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return alarmTemplateService.batchRemove(param.getIds());
    }
}
