package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.intf.AnalysisRuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 监控分析规则Controller
 *
 * @author zhangdi
 * @since Aug 15,2019
 */

@RestController
@Tag(name = "监控分析规则")
@RequestMapping("/v1/analysis/rules")
public class AnalysisRuleController {

    private final AnalysisRuleService analysisRuleService;

    @Autowired
    public AnalysisRuleController(AnalysisRuleService analysisRuleService) {
        this.analysisRuleService = analysisRuleService;
    }

    /**
     * 监控分析规则列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 数据列表
     */
    @GetMapping
    @Operation(summary = "监控分析规则列表")
    public GeneralResult list(Pager pager,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.list(pager, context);
    }

    /**
     * 新增监控分析规则
     *
     * @param analysisRule 监控分析规则
     * @param context      当前用户上下文
     * @return 是否创建成功
     */
    @PostMapping
    @Operation(summary = "创建监控分析规则")
    public GeneralResult create(@RequestBody MonitorAnalysisRule analysisRule,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.create(analysisRule, context);
    }

    /**
     * 修改监控分析规则
     *
     * @param analysisRule 监控分析规则
     * @param context      当前用户上下文
     * @return 是否修改成功
     */
    @PutMapping("/{id}")
    @Operation(summary = "监控分析规则修改")
    public GeneralResult modify(@RequestBody MonitorAnalysisRule analysisRule,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.modify(analysisRule, context);
    }

    /**
     * 删除监控分析规则
     *
     * @param id      规则id
     * @param context 当前用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "监控分析规则删除")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return analysisRuleService.remove(id, context.getTarget());
    }

    /**
     * 监控分析规则详情展示
     *
     * @param id 规则id
     * @return 分析详情
     */
    @GetMapping("/{id}")
    @Operation(summary = "监控分析规则详情")
    public GeneralResult detail(@PathVariable(value = Common.ID) Long id) {
        return analysisRuleService.detail(id);
    }

}
