package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.TopService;
import com.bocloud.cms.model.LineChartResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

@Tag(name = "对外提供接口")
@RestController
@RequestMapping("/v1/api")
@Slf4j
public class ApiController {

    @Autowired
    private TopService topService;

    @GetMapping("/top")
    @Operation(summary = "查询TOP")
    public List<TopModel> top(@RequestParam(value = "type", required = false) String type,
                              @RequestParam(value = "vendorId", required = false) Long vendorId,
                              @RequestParam(value = "vendorType", required = false) String vendorType,
                              @RequestParam(value = "limit", required = false, defaultValue = "5") Integer limit,
                              @Value(Common.REQ_CONTEXT) RequestContext context,
                              HttpServletRequest request) {
        LineChartResult chart = switch (type) {
            case "hostCpu" -> this.topService.topOfHostCpu(vendorId, limit);
            case "hostMem" -> this.topService.topOfHostMem(vendorId, limit);
            case "vmCpu" -> this.topService.topOfvmCpu(context, vendorId, limit, request, vendorType);
            case "vmMem" -> this.topService.topOfvmMem(context, vendorId, limit, request, vendorType);
            default -> throw new IllegalArgumentException("unknow type " + type);
        };
        List<TopModel> models = new ArrayList<>();
        if (chart == null) {
            return models;
        }
        for (int i = 0; i < chart.getKeys().size(); i++) {
            models.add(new TopModel(chart.getKeys().get(i), chart.getValues()[0].getData().get(i).toString()));
        }
        return models;
    }

    @Data
    @AllArgsConstructor
    public static class TopModel {
        private final String name;
        private final String value;
    }

}
