package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.*;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * HCSO云监控
 *
 * @author zhangyf
 * @since 2022-07-06
 */
@Tag(name = "HCSO云监控")
@RestController
@RequestMapping("/v1/hcso")
public class HcsoController {

    @Autowired
    HuaweiVmService huaweiVmService;
    @Autowired
    HcsoVolumeService hcsoVolumeService;
    @Autowired
    HcsoMySqlService hcsoMySqlService;
    @Autowired
    HcsoPostgreSqlService hcsoPostgreSqlService;
    @Autowired
    HcsoNatService hcsoNatService;
    @Autowired
    HcsoElbService hcsoElbService;
    @Autowired
    HcsoElbListenerService hcsoElbListenerService;
    @Autowired
    HcsoEipService hcsoEipService;

    /**
     * 云硬盘列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return 云硬盘列表
     */
    @GetMapping("/volumes")
    @Operation(summary = "云硬盘列表")
    public GeneralResult listOfVolumes(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoVolumeService.listOfVolume(pager, context, request);
    }

    /**
     * MySQL数据库列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return MySQL数据库列表
     */
    @GetMapping("/mysql")
    @Operation(summary = "MySQL数据库列表")
    public GeneralResult listOfMySql(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoMySqlService.listRds(pager, context, request);
    }

    /**
     * Postgre数据库列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return Postgre数据库列表
     */
    @GetMapping("/postgresql")
    @Operation(summary = "Postgre数据库列表")
    public GeneralResult listOfPostgre(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoPostgreSqlService.listRds(pager, context, request);
    }

    /**
     * Nat网关列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return Nat网关列表
     */
    @GetMapping("/natgateway")
    @Operation(summary = "Nat网关列表")
    public GeneralResult listOfNatgateway(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoNatService.listOfNat(pager, context, request);
    }

    /**
     * 负载均衡器列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return 负载均衡器列表
     */
    @GetMapping("/lbs")
    @Operation(summary = "负载均衡器列表")
    public GeneralResult listOfLoadBalancer(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoElbService.listOfLoadBalancer(pager, context, request);
    }

    /**
     * 负载均衡监听器列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return 负载均衡监听器列表
     */
    @Deprecated
    @GetMapping("/lb/listeners")
    @Operation(summary = "负载均衡监听器列表")
    public GeneralResult listOfLbListener(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoElbListenerService.listOfLbListener(pager, context, request);
    }

    /**
     * 弹性IP列表
     *
     * @param pager   分页
     * @param context 用户上下文
     * @param request 请求request
     * @return 弹性IP列表
     */
    @GetMapping("/eips")
    @Operation(summary = "弹性IP列表")
    public GeneralResult listOfEip(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return hcsoEipService.listOfEip(pager, context, request);
    }

}