package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.HmcService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * HMC监控
 *
 * @author zhangdi
 * @since Oct 21,2019
 */

@RestController
@Tag(name = "Hmc监控")
@RequestMapping("/v1/hmc")
public class HmcController {

    @Autowired
    private HmcService hmcService;

    /**
     * Hmc主机列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 主机列表
     */
    @GetMapping("/servers")
    @Operation(summary = "Hmc主机列表展示")
    public GeneralResult listOfServer(Pager pager,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hmcService.listOfServer(pager, context);
    }

    /**
     * Hmc分区列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 分区列表
     */
    @GetMapping("/partitions")
    @Operation(summary = "Hmc分区列表展示")
    public GeneralResult listOfPartition(Pager pager,
                                         @Value(Common.REQ_CONTEXT) RequestContext context) {
        return hmcService.listOfPartition(pager, context);
    }
}