package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.KubernetesService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * Kubernetes监控
 *
 * @author zhangdi
 * @since Sept 3,2019
 */

@RestController
@Tag(name = "Kubernetes监控")
@RequestMapping("/v1/kubernetes")
public class KubernetesController {

    @Autowired
    private KubernetesService kubernetesService;

    /**
     * 节点列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 节点列表
     */
    @GetMapping("/nodes")
    @Operation(summary = "节点列表")
    public GeneralResult listOfNode(Pager pager,
                                    @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kubernetesService.listOfNode(pager, context);
    }

    /**
     * 容器组列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 容器组列表
     */
    @GetMapping("/pods")
    @Operation(summary = "容器组列表")
    public GeneralResult listOfPod(Pager pager,
                                   @Value(Common.REQ_CONTEXT) RequestContext context) {
        return kubernetesService.listOfPod(pager, context);
    }

    /**
     * 节点Node基本信息
     *
     * @param id
     * @return 节点Node基本信息
     */
    @GetMapping("/nodes/overview")
    @Operation(summary = "节点基本信息")
    public GeneralResult nodeOverview(Long id) {
        return kubernetesService.detailOfNode(id);
    }

    /**
     * 容器组基本信息
     *
     * @param name
     * @return 容器组基本信息
     */
    @GetMapping("/pods/overview")
    @Operation(summary = "容器组基本信息")
    public GeneralResult podOverview(String name) {
        return kubernetesService.detailOfPod(name);
    }


}
