package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.intf.MetricService;
import com.bocloud.cms.model.DeleteIdList;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 监控指标
 *
 * @author zhangdi
 * @version 5.6.0
 * @updateDate 2022/2/21 18:01 fanzhenxi 增加CRUD功能并可与监控资源类型关联
 * @since May 28,2019
 */
@Tag(name = "监控指标")
@RestController
@RequestMapping("/v1/metrics")
class MetricController {

    @Autowired
    private MetricService metricService;

    /**
     * 查询监控指标列表 获取到的指标为开启状态
     *
     * @param resourceType 资源类型
     * @return 指标列表
     */
    @GetMapping
    @Operation(summary = "监控指标列表")
    public GeneralResult list(String resourceType) {
        return metricService.list(resourceType);
    }

    /**
     * 监控配置展示指标列表
     *
     * @param pager 分页
     * @return 指标列表
     */
    @GetMapping("/monitor/list")
    @Operation(summary = "监控指标列表")
    public GeneralResult monitorMetricList(Pager pager) {
        return metricService.monitorMetricList(pager);
    }

    /**
     * 监控指标新增
     *
     * @param metric  新增指标参数
     * @param context 当前用户上下文
     * @return 是否新增成功
     */
    @PostMapping
    @Operation(summary = "新增监控指标")
    public GeneralResult create(@RequestBody Metric metric,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.create(metric, context);
    }

    /**
     * 监控指标修改
     *
     * @param metric  修改的监控指标
     * @param context 当前用户上下文
     * @return 是否修改成功
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改监控指标")
    public GeneralResult modify(@RequestBody Metric metric,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.modify(metric, context);
    }

    /**
     * 监控指标删除
     *
     * @param id      指标id
     * @param context 当前用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除监控指标")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.remove(id, context);
    }

    /**
     * 批量删除监控指标
     *
     * @param param   批量删除指标参数id
     * @param context 当前用户上下文
     * @return 是否删除成功
     */
    @DeleteMapping
    @Operation(summary = "批量删除监控指标")
    public GeneralResult batchRemove(@RequestBody DeleteIdList param,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return metricService.batchRemove(param.getIds(), context);
    }

    /**
     * 获取该资源类型下所有status为开启的指标列表
     *
     * @param resourceType
     * @return 指标状态列表
     */
    @GetMapping("/state/ids")
    @Operation(summary = "获取该资源类型下所有状态为开启的指标列表")
    public GeneralResult getByResourceType(String resourceType) {
        return metricService.getByResourceType(resourceType);
    }
}