package com.bocloud.cms.booter.controller;

import com.bocloud.cms.entity.MonitorResourceType;
import com.bocloud.cms.intf.MonitorResourceTypeService;
import com.bocloud.cms.model.DeleteIdList;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * 告警配置资源类型
 *
 * @author fanzhenxi
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@RestController
@Tag(name = "监控类型")
@RequestMapping("/v1/monitor/resourcetype")
public class MonitorResourceTypeController {

    @Autowired
    private MonitorResourceTypeService monitorResourceTypeService;

    /**
     * 新增监控资源类型
     *
     * @param monitorResourceType 监控类型对象
     * @param context             当前用户上下文
     * @return 成功/失败
     */
    @PostMapping
    @Operation(summary = "新增监控资源类型")
    public GeneralResult create(@RequestBody MonitorResourceType monitorResourceType,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.create(monitorResourceType, context);
    }

    /**
     * 获取监控资源类型列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 监控资源类型列表
     */
    @GetMapping
    @Operation(summary = "获取监控资源类型列表")
    public GeneralResult list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.list(pager, context);
    }

    /**
     * 修改监控资源类型
     *
     * @param monitorResourceType 监控类型对象
     * @param context             当前用户上下文
     * @return 成功/失败
     */
    @PutMapping("/{id}")
    @Operation(summary = "修改监控资源类型")
    public GeneralResult modify(@PathVariable(value = Common.ID) Long id,
                                @RequestBody MonitorResourceType monitorResourceType,
                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.modify(id, monitorResourceType, context);
    }

    /**
     * 删除监控资源类型
     *
     * @param id      监控类型对象id
     * @param context 当前用户上下文
     * @return 成功/失败
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "删除监控资源类型")
    public GeneralResult remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.remove(id, context);
    }

    /**
     * 批量删除监控资源类型
     *
     * @param deleteIdList 监控类型对象id列表
     * @param context      当前用户上下文
     * @return 成功/失败 个数
     */
    @DeleteMapping("/remove")
    @Operation(summary = "批量删除监控资源类型")
    public GeneralResult batchRemove(@RequestBody DeleteIdList deleteIdList,
                                     @Value(Common.REQ_CONTEXT) RequestContext context) {
        return monitorResourceTypeService.batchRemove(deleteIdList.getIds(), context.getTarget());
    }

    /**
     * 资源类型绑定监控指标操作
     *
     * @param monitorResourceType 指标资源类型
     * @param requestContext      request上下文
     * @return 成功/失败
     */
    @Operation(summary = "资源类型绑定监控指标")
    @PostMapping("/binding/metrics")
    public Result bindingPlugins(@RequestBody MonitorResourceType monitorResourceType,
                                 @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return monitorResourceTypeService.bindingMetrics(monitorResourceType.getMetricIds(), monitorResourceType.getResourceType(), requestContext);
    }
}
