package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusResult;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Set;

/**
 * 查询prometheus监控数据
 *
 * @author zhangyf
 * @since 2019-09-06
 */

@RestController
@Tag(name = "prometheus")
@RequestMapping("/v1/prometheus")
@Slf4j
public class PrometheusController {


    private final PrometheusService prometheusService;

    @Autowired
    public PrometheusController(PrometheusService prometheusService) {
        this.prometheusService = prometheusService;
    }

    /**
     * prometheus监控数据查询
     *
     * @return 数据结果查询
     */
    @GetMapping
    @Operation(summary = "prometheus监控数据查询")
    public GeneralResult query(@RequestParam(value = "expr") String expr,
                               @RequestParam(value = "time", required = false) String time,
                               @RequestParam(value = "start", required = false) String start,
                               @RequestParam(value = "end", required = false) String end,
                               @RequestParam(value = Common.SIMPLE, required = false, defaultValue = "false") Boolean simple,
                               @RequestParam(value = "step", required = false) String step) {
        try {
            if (simple) {
                Object value = prometheusService.simpleQuery(expr, time);
                return new GeneralResult<>(true, value, "success");
            }
            List<PrometheusResult> results;
            if (StringUtils.isBlank(start)) {
                results = prometheusService.query(expr, time);
            } else {
                results = prometheusService.queryRange(expr, start, end, step);
            }
            return new GeneralResult<>(true, results, "success");
        } catch (RuntimeException e) {
            log.error(e.getMessage(), e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

    /**
     * 过滤条件集合 用于图表的下拉框
     *
     * @param expr   查询条件
     * @param filter 过滤条件
     * @return 数据展示
     */
    @GetMapping("/filter")
    @Operation(summary = "prometheus过滤条件集合")
    public GeneralResult filter(@RequestParam(value = "expr") String expr,
                                @RequestParam(value = "filter") String filter) {
        Set<String> set = prometheusService.filter(expr, filter);
        return new GeneralResult<>(true, set, "success");
    }
}
