package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.ServiceService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 服务Controller
 *
 * @author zhangdi
 * @since Feb 28,2020
 */

@RestController
@Tag(name = "OpenStack服务监控")
@RequestMapping("/v1/services")
public class ServiceController {

    private final ServiceService serviceService;

    @Autowired
    public ServiceController(ServiceService serviceService) {
        this.serviceService = serviceService;
    }

    /**
     * 服务列表
     *
     * @param context 当前用户上下文
     * @return OpenStack服务列表
     */
    @GetMapping
    @Operation(summary = "OpenStack服务列表")
    public GeneralResult list(@RequestParam(value = "vendorId", required = false) Long vendorId,
                              @RequestParam(value = "name", required = false) String name,
                              @RequestParam(value = "status", required = false) String status,
                              @Value(Common.REQ_CONTEXT) RequestContext context) {
        return serviceService.list(name, status, vendorId, context);
    }

}
