package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.StorageService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 华为存储监控
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@RestController
@Tag(name = "存储监控")
@RequestMapping("/v1")
public class StorageController {
    @Autowired
    private StorageService storageService;

    /**
     * 存储设备列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 存储设备列表
     */
    @GetMapping("/storages")
    @Operation(summary = "存储设备列表")
    public GeneralResult listDevices(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listDevices(pager, context, request);
    }

    /**
     * 硬盘列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 硬盘列表
     */
    @GetMapping("/disks")
    @Operation(summary = "硬盘列表")
    public GeneralResult listDisks(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listDisks(pager, context, request);
    }

    /**
     * 存储卷列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 存储卷列表
     */
    @GetMapping("/volumes")
    @Operation(summary = "存储卷列表")
    public GeneralResult listVolumes(Pager pager, @RequestParam(value = "action", required = false) String action,
                                     @Value(Common.REQ_CONTEXT) RequestContext context, HttpServletRequest request) {
        return storageService.listVolumes(pager, context, action, request);
    }
}
