package com.bocloud.cms.booter.controller;

import com.bocloud.cms.intf.TopService;
import com.bocloud.cms.model.LineChartResult;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 监控TOP
 *
 * @author zhangyf
 * @since 2018-12-27
 */
@Tag(name = "监控TOPN")
@RestController
@RequestMapping("/v1/tops")
@Slf4j
public class TopController {


    @Autowired
    private TopService topService;

    /**
     * 宿主机、云主机Top5
     *
     * @param type     类型
     * @param vendorId 平台id
     * @param limit    分页展示数据
     * @param context  当前用户上下文
     * @return 数据展示
     */
    @GetMapping
    @Operation(summary = "宿主机/云主机TopN图表")
    public GeneralResult top(@RequestParam(value = "type", required = false) String type,
                             @RequestParam(value = "vendorId", required = false) Long vendorId,
                             @RequestParam(value = "vendorType", required = false) String vendorType,
                             @RequestParam(value = "limit", required = false, defaultValue = "5") Integer limit,
                             @Value(Common.REQ_CONTEXT) RequestContext context,
                             HttpServletRequest request) {
        LineChartResult chart = null;
        try {
            switch (type) {
                case "hostCpu":
                    chart = this.topService.topOfHostCpu(vendorId, limit);
                    break;
                case "hostMem":
                    chart = this.topService.topOfHostMem(vendorId, limit);
                    break;
                case "vmCpu":
                    chart = this.topService.topOfvmCpu(context, vendorId, limit, request, vendorType);
                    break;
                case "vmMem":
                    chart = this.topService.topOfvmMem(context, vendorId, limit, request, vendorType);
                    break;
                default:
                    throw new IllegalArgumentException("unknow type " + type);
            }
            return new GeneralResult<>(true, chart, "success");
        } catch (RuntimeException e) {
            log.error(e.getMessage(), e);
            return new GeneralResult<>(false, "获取Top监控数据失败:" + e.getMessage());
        }
    }
}