package com.bocloud.cms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.enums.ResourceCategory;
import com.bocloud.cms.intf.CcsService;
import com.bocloud.cms.intf.CmsService;
import com.bocloud.cms.intf.KvsService;
import com.bocloud.cms.intf.RdsService;
import com.bocloud.cms.service.internal.mms.MmsCcsService;
import com.bocloud.cms.service.internal.mms.MmsCmsService;
import com.bocloud.cms.service.internal.mms.MmsKvsService;
import com.bocloud.cms.service.internal.mms.MmsRdsService;
import com.bocloud.pms.entity.Ccs;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.Rds;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * SoftwareEventListenner
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Component
@Slf4j
public class SoftwareEventListenner implements MessageListener {

    @Autowired
    private MmsKvsService mmsKvsService;
    @Autowired
    private MmsRdsService mmsRdsService;
    @Autowired
    private MmsCmsService mmsCmsService;
    @Autowired
    private MmsCcsService mmsCcsService;
    @Autowired
    private KvsService kvsService;
    @Autowired
    private RdsService rdsService;
    @Autowired
    private CmsService cmsService;
    @Autowired
    private CcsService ccsService;

    @RabbitListener(queues = "software.event.queue", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        log.info("安装日志{}", message);
        try {
            String content = new String(message.getBody());
            JSONObject jsonObject = JSONObject.parseObject(content);
            JSONObject data = jsonObject.getJSONObject("data");
            if ("create".equalsIgnoreCase(jsonObject.getString("action"))) {
                this.create(data);
            } else if ("delete".equalsIgnoreCase(jsonObject.getString("action"))) {
                this.delete(data);
            } else {
                log.error("illegal argument " + jsonObject.getString("action"));
            }
        } catch (Exception e) {
            log.error("parse software receive exception!", e);
        }
    }

    /**
     * 资源创建完成后执行的操作
     *
     * @param data
     * @throws Exception
     */
    private void create(JSONObject data) throws Exception {
        Long id = data.getLong("id");
        ResourceCategory category = ResourceCategory.valueOf(data.getString("type"));
        switch (category) {
            case MySQL:
            case SqlServer:
            case PostgreSQL:
                rdsService.init(mmsRdsService.detail(id));
                break;
            case Redis:
            case MongoDB:
            case MemCache:
                kvsService.init(mmsKvsService.detail(id));
                break;
            case Kafka:
            case RabbitMQ:
                cmsService.init(mmsCmsService.detail(id));
                break;
            case Zookeeper:
            case Etcd:
                ccsService.init(mmsCcsService.detail(id));
                break;
            default:
                log.warn("unsupported resource category " + category.name().toLowerCase());
                break;
        }
    }

    /**
     * 资源被删除、退订后删除对应配置文件
     *
     * @param data
     */
    private void delete(JSONObject data) throws Exception {
        Long id = data.getLong("id");
        ResourceCategory category = ResourceCategory.valueOf(data.getString("type"));
        switch (category) {
            case MySQL:
            case SqlServer:
            case PostgreSQL:
                Rds rds = new Rds();
                rds.setId(id);
                rds.setType(category.name());
                rdsService.destroy(rds);
                break;
            case Redis:
            case MongoDB:
            case MemCache:
                Kvs kvs = new Kvs();
                kvs.setId(id);
                kvs.setType(category.name());
                kvsService.destroy(kvs);
                break;
            case Kafka:
            case RabbitMQ:
                Cms cms = new Cms();
                cms.setId(id);
                cms.setType(category.name());
                cmsService.destroy(cms);
                break;
            case Zookeeper:
            case Etcd:
                Ccs ccs = new Ccs();
                ccs.setId(id);
                ccs.setType(category.name());
                ccsService.destroy(ccs);
                break;
            default:
                log.warn("unsupported resource category " + category.name().toLowerCase());
                break;
        }
    }

}
