package com.bocloud.cms.entity;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.utils.DateSerializer;
import com.megatron.entity.annotations.*;
import com.megatron.entity.bean.Generic;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.List;

/**
 * 告警资源类型表
 *
 * @author fanzx
 * @version 5.6.0
 * @since 2022/2/16
 */
@Data
@EqualsAndHashCode(callSuper = false)
@AllArgsConstructor
@NoArgsConstructor
@Table("monitor_resource_type")
public class MonitorResourceType extends Generic {

    /**
     * id
     */
    @PK(value = PKStrategy.AUTO)
    private Long id;

    /**
     * 资源名称
     *  对应的是CloudProvider中枚举类型
     */
    @Column("name")
    private String name;

    /**
     * 资源类型
     *  对应monitor_metric中的resourceType 一对多
     */
    @Column("resource_type")
    private String resourceType;

    /**
     * 是否在租户端展示true展示/false不展示
     */
    @Column("is_tenant")
    private Boolean isTenant;

    /**
     * 创建时间
     */
    @Column("gmt_create")
    @IgnoreUpdate
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtCreate;

    /**
     * 修改时间
     */
    @Column("gmt_modify")
    @JsonSerialize(using = DateSerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gmtModify;

    /**
     * 是否删除true删除/false未删除
     */
    @Column("is_deleted")
    private Boolean isDeleted;

    /**
     * 描述
     */
    @Column("remark")
    private String remark;

    /**
     * metric指标列表
     */
    @IgnoreAll
    private List<Integer> metricIds;
}
