package com.bocloud.ims.entity;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DateDeserializer;
import com.megatron.common.utils.DateSerializer;
import com.megatron.common.utils.DaySerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;
import java.util.concurrent.Future;

/**
 * @author  hya
 * @date 2022/5/12 16:37
 */
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@Table("cloud_rds")
public class CloudRds extends GenericEntity {
    @PK(value = PKStrategy.AUTO)
    private Long id; // ID
    @Column("uuid")
    private String uuid;
    @Column("is_freeze")
    private Boolean freeze;
    @Column("cpu")
    private Integer cpu;
    @Column("memory")
    private Integer memory;
    @Column("port")
    private Integer port;
    @Column("flavor")
    private String flavor;
    @Column("switch_strategy")
    private String switchStrategy;
    @Column("replication_mode")
    private String replicationMode;
    @Column("maintenance_window")
    private String maintenanceWindow;
    @Column("private_ips")
    private String privateIps;
    @Column("public_ips")
    private String publicIps;
    @Column("db_time_zone")
    private String dBTimeZone;
    @Column("storage_used")
    private Double storageUsed;
    @Column("storage_size")
    private Integer storageSize;
    @Column("storage_type")
    private String storageType;
    @Column("db_nodes")
    private String dbNodes;
    @Column("db_type")
    private String dbType;
    @Column("engine")
    private String engine;
    @Column("version")
    private String version;
    @Column("complete_version")
    private String completeVersion;
    @Column("enterprise_project_id")
    private Long enterpriseProjectId;
    @Column("region_id")
    private String regionId;
    @Column("vendor_id")
    private Long vendorId;
    @Column("vpc_id")
    private Long vpcId;
    @Column("subnet_id")
    private Long subnetId;
    @Column("security_group_id")
    private Long securityGroupId;
    @Column("create_time")
    @JsonSerialize(using = DateSerializer.class)
    @JsonDeserialize(using = DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;
    @Column("expired_time")
    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;// 过期时间
    @Column("tenant_id")
    private Long tenantId;// 租户ID
    @Column("catalog")
    private RequestContext.Catalog catalog;// 账号类别
    @Column("sn")
    private String sn;// 订单号
    @Column("category_ids")
    private String categoryIds;// 服务目录
    @Column("backup_policy")
    private String backupPolicy;

    @IgnoreAll
    private String regionName;
    @IgnoreAll
    private String vpcName;
    @IgnoreAll
    private String subnetName;
    @IgnoreAll
    private String subnetUuid;
    @IgnoreAll
    private String securityGroupName;
    @IgnoreAll
    private String subnetCidr;
    @IgnoreAll
    private String enterpriseProjectName;
    @IgnoreAll
    private String password;
    @IgnoreAll
    private String opsStartTime;
    @IgnoreAll
    private String opsEndTime;
    @IgnoreAll
    private String availabilityZone;
    @IgnoreAll
    private Long configurationId;
    @IgnoreAll
    private String ipRange;
    @IgnoreAll
    private String tenantName;
    @IgnoreAll
    private Future<GeneralResult> future;

    public CloudRds(Future<GeneralResult> future) {
        this.future = future;
    }
}