package com.bocloud.ims.entity;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import com.megatron.common.encrypt.Encryptor;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.DaySerializer;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * @ClassName: ContainerCluster
 * @Description: 容器Kubernetes 集群
 * @Author lxy
 * @Date 2022/7/14 17:54
 */
@Slf4j
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
@Table("container_cluster")
public class ContainerCluster extends GenericEntity {
    @PK(PKStrategy.AUTO)
    private Long id;// ID
    @Column("region_id")
    private String regionId;// 所属域

    @Column("vendor_id")
    private Long vendorId;// 所属云平台
    @Column("cluster_uuid")
    private String clusterUuid;// 集群ID
    @Column("cluster_type")
    private String clusterType;// 集群类型
    // 集群创建时间
    @Column("created")
    private Date created;
    // 集群版本
    @Column("version")
    private String version;
    // 集群Docker版本。
    @Column("docker_version")
    private String dockerVersion;
    // 集群API Server访问地址，包含内网访问地址以及公网访问地址。
    @Column("master_url")
    private String masterUrl;
    // 集群安全组ID。
    @Column("security_group_uuid")
    private String securityGroupUuid;
    /**
     * 集群运行状态，取值： initial：集群创建中。 failed：集群创建失败。 running：集群运行中。 updating：集群升级中。 updating_failed：集群升级失败。 scaling：集群伸缩中。
     * stopped：集群已经停止运行。 deleting：集群删除中。 deleted：集群已经被删除。 delete_failed：集群删除失败。
     */
    // @Column("status")
    // private String status;

    /**
     * Pod网络地址段，必须是有效的私有网段，即以下网段及其子网：
     * <p>
     * 10.0.0.0/8 172.16-31.0.0/12-16 192.168.0.0/16 不能与VPC及VPC内已有Kubernetes集群使用的网段重复，创建成功后不能修改。
     */
    @Column("subnet_cidr")
    private String subnetCidr;
    // 集群专有网络ID，创建集群时必传。
    @Column("vpc_uuid")
    private String vpcUuid;
    @IgnoreAll
    private String vpcName;
    // 虚拟交换机ID，创建集群时可选择1~3个虚拟交换机。为保证集群高可用，建议选择不同可用区的虚拟交换机。
    @Column("vswitch_uuid")
    private String vswitchUuid;
    // 集群所属可用区ID。
    @Column("zone_id")
    private String zoneId;
    // 托管版集群类型，面向托管集群： ack.pro.small：专业托管集群。 ack.standard：标准托管集群。
    @Column("cluster_spec")
    private String clusterSpec;
    // 集群删除保护
    @Column("deletion_protection")
    private Boolean deletionProtection;
    @Column("eip_uuid")
    private String eipUuid;
    @Column("load_balancer_uuid")
    private String loadBalancerUuid;
    @Column("resource_group_uuid")
    private String resourceGroupUuid;// 资源组id
    // SSH登录密码。密码规则为8~30 个字符，且至少同时包含三项（大小写字母、数字和特殊符号），和key_pair二选一。
    @Column("login_password")
    public String loginPassword;
    // 密钥对名称，和login_password二选一。
    @Column("key_pair")
    public String keyPair;

    // 网络插件
    @Column("capabilities_network")
    private String capabilitiesNetwork;
    // kube-proxy代理模式
    @Column("capabilities_proxyModel")
    private String capabilitiesProxyModel;

    // HSC START
    // 子网uuid
    @Column("subnet_uuid")
    private String subnetUuid;
    @IgnoreAll
    private String subnetName;
    // 容器网段
    @Column("container_cidr")
    private String containerCidr;
    // 服务网段
    @Column("service_cidr")
    private String serviceCidr;
    // HCS END

    // 自服务 start
    @Column("tenant_id")
    private Long tenantId;// 租户ID
    @IgnoreAll
    private String tenantName; // 租户名称

    @Column("project_id")
    private Long projectId;// 项目ID
    @IgnoreAll
    private String projectName; // 项目名称

    @Column("uuid")
    private String uuid;// uuid，对应子订单号

    @Column("expired_time")
    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date expiredTime;// 过期时间
    // 所属申请类别
    @Column("catalog")
    private RequestContext.Catalog catalog;
    // 所属业务id
    @Column("business_id")
    private Long businessId;
    // 所属业务名称
    private String businessName;

    @Column("is_recycle")
    private Boolean isRecycle = false;
    @Column("recycle_time")
    @JsonSerialize(using = DaySerializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date recycleTime;// 回收时间
    // 自服务 end

    @IgnoreAll
    private String regionName;
    @IgnoreAll
    private String vpcSafetyType;//安全类型
    @IgnoreAll
    private String vpcFunctionType;
    @IgnoreAll
    private String applicationName; //应用系统
    @IgnoreAll
    private String resourceSetName; //资源集
    @IgnoreAll
    private String imageId;//自定义镜像




    public void preSave(Long vendorId, String regionId, Long userId) {
        this.setGmtModify(new Date());
        this.setVendorId(vendorId);
        this.setRegionId(regionId);
        this.setCreatorId(userId);
        this.setMenderId(userId);
        this.setGmtCreate(new Date());
        this.setTenantId(0L);
        this.setProjectId(0L);
    }

    public void preUpdate(Long vendorId, String regionId, Long userId) {
        this.setGmtModify(new Date());
        this.setVendorId(vendorId);
        this.setMenderId(userId);
        this.setRegionId(regionId);
    }


    /**
     * 云主机 是否开启告警状态
     */
    private Boolean alarmEnable;
    private String cpuUsage;
    private String memUsage;
}
