package com.bocloud.ims.entity.kubernetes;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.megatron.entity.annotations.Column;
import com.megatron.entity.annotations.IgnoreAll;
import com.megatron.entity.annotations.PK;
import com.megatron.entity.annotations.Table;
import com.megatron.entity.bean.GenericEntity;
import com.megatron.entity.meta.PKStrategy;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.Date;

/**
 * k8s容器组
 *
 * @author weiwei
 */
@Table("kube_pod")
@Data
public class KubePod extends GenericEntity {

    @PK(value = PKStrategy.AUTO)
    private Long id;
    @Column("instance_id")
    private String instanceId;
    @Column("pod_ip")
    private String podIp;
    @Column("labels")
    private String labels;
    /**
     * 命名空间ID
     */
    @Column("ns_id")
    private Long nsId;
    @Column("ns_name")
    private String namespace;
    /**
     * 节点ID
     */
    @Column("node_id")
    private Long nodeId;
    @Column("node_name")
    private String nodeName;
    /**
     * 服务ID
     */
    @Column("service_id")
    private Long serviceId;
    @Column("service_name")
    private String serviceName;
    @Column("vendor_id")
    private Long vendorId;
    @Column("yaml_text")
    private String yamlText;
    @Column("deployment_name")
    private String deploymentName;
    @Column("deployment_uid")
    private String deploymentUid;
    @Column("deployment_api_version")
    private String deploymentApiVersion;
    @Column("tenant_id")
    private Long tenantId;
    @Column("project_id")
    private Long projectId;
    @Column("uid")
    private String uid;
    @Column("app_id")
    private Long appId;
    @Column("app_name")
    private String appName;
    @Column("kind")
    private String kind;
    @Column("api_version")
    private String apiVersion;
    @Column("image_name")
    private String imageName;

    @Column("creation_time")
    @JsonSerialize(using = com.megatron.common.utils.DateSerializer.class)
    @JsonDeserialize(using = com.megatron.common.utils.DateDeserializer.class)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date creationTime;

    /**
     * 所属租户
     */
    @IgnoreAll
    private String tenantName;
    @IgnoreAll
    private String projectVisibility;

    @Column("parent_resource_uid")
    private String parentResourceUid;
    @Column("parent_resource_type")
    private String parentResourceType;
    @Column("parent_resource_name")
    private String parentResourceName;
    @Column("parent_resource_version")
    private String parentResourceVersion;

}
