package com.bocloud.cms.repository;

import com.bocloud.cms.entity.AlarmTemplate;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 告警模板
 *
 * @author fanzx
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Repository
public class AlarmTemplateRepository extends BasicGenericDao<AlarmTemplate, Long> {

    public AlarmTemplateRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 总数
     *
     * @param params  参数集合
     * @param context 当前用户上下文
     * @return 总数
     */
    public int count(List<Param> params, RequestContext context) {
        String sql = "select count(1) from monitor_alarm_template where is_deleted = 0 and tenant_id = :tenantId  ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("tenantId", context.getTenant());
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 列表
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 列表
     */
    public List<AlarmTemplate> list(Pager pager, RequestContext context) {
        String sql = "select t.* from monitor_alarm_template t where t.is_deleted = 0 and t.tenant_id = :tenantId ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("tenantId", context.getTenant());
        sql = getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(AlarmTemplate.class, sql, paramMap);
    }

    /**
     * 按类型获取告警模版
     *
     * @param resourceType 资源类型
     * @param tenantId     租户ID
     * @return 告警模版
     */
    public AlarmTemplate getByType(String resourceType, Long tenantId) {
        String sql = "select * from monitor_alarm_template where is_deleted = 0 and resource_type = :resourceType and tenant_id = :tenantId ORDER BY gmt_create desc ";
        Map<String, Object> paramMap = MapTools.simpleMap("resourceType", resourceType);
        paramMap.put("tenantId", tenantId);
        List<AlarmTemplate> alarmTemplates = this.list(AlarmTemplate.class, sql, paramMap);
        return ListTool.isEmpty(alarmTemplates) ? null : alarmTemplates.get(0);
    }

}
