package com.bocloud.cms.repository;

import com.bocloud.cms.entity.MonitorAuth;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 云平台监控运维认证信息
 *
 * @author lyy
 * @version 5.5.0
 * @date 2022/2/21 18:01
 */
@Repository
public class AuthRepository extends BasicGenericDao<MonitorAuth, Long> {

    public AuthRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据云平台ID查询云平台监控运维认证信息
     *
     * @param vendorId 云平台ID
     * @return 云平台监控运维认证信息
     */
    public MonitorAuth queryByVendorId(Long vendorId) {
        String sql = "select * from monitor_auth where vendor_id = :vendorId ";
        Map<String, Object> paramMap = MapTools.simpleMap("vendorId", vendorId);
        List<MonitorAuth> auth = this.list(MonitorAuth.class, sql, paramMap);
        return ListTool.isEmpty(auth) ? null : auth.get(0);
    }

    public List<MonitorAuth> listAll() {
        String sql = "select * from monitor_auth ";
        return this.list(MonitorAuth.class, sql);
    }
}
