package com.bocloud.cms.repository;

import com.bocloud.cms.entity.ResourceRelation;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 告警规则绑定资源关系
 *
 * @author fanzx
 * @date 2021/09/18
 */
@Repository
public class ResourceRelationRepository extends BasicGenericDao<ResourceRelation, Long> {

    public ResourceRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据ruleGroupId查询对象集合
     *
     * @param id 告警策略ID
     * @return 告警规则绑定资源关系集合
     */
    public List<ResourceRelation> listByRuleGroupId(Long id) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        return this.list(ResourceRelation.class, sql, paramMap);
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *
     * @param id 告警策略ID
     * @return 是否成功
     */
    public boolean delete(Long id) {
        String sql = "update monitor_resource_relation set is_deleted = 1, gmt_modify = :gmtModify where group_id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }


    /**
     * 根据ruleGroupID 去删除monitor_resource_relation表信息
     *
     * @param id 告警策略ID
     */
    public void deleteByGroupId(Long id) {
        String sql = "delete from monitor_resource_relation  where group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        this.execute(sql, paramMap);
    }

    /**
     * 根据ruleGroupId,和resourceId查询出单条ResourceRelation对象
     *
     * @param ruleGroupId 告警策略ID
     * @param resourceId  监控资源ID
     * @return 告警规则绑定资源关系
     */
    public ResourceRelation find(Long ruleGroupId, String resourceId) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and target = :target and group_id = :groupId ";
        Map<String, Object> paramMap = MapTools.simpleMap("target", resourceId);
        if (null != ruleGroupId) {
            paramMap.put("groupId", ruleGroupId);
        }
        List<ResourceRelation> ruleGroups = this.list(ResourceRelation.class, sql, paramMap);
        return ListTool.isEmpty(ruleGroups) ? null : ruleGroups.get(0);
    }

    /**
     * 根据resourceId,去查询到所关联的ruleGroupId集合
     *
     * @param resourceId 监控资源ID
     * @return 告警规则绑定资源关系集合
     */
    public List<ResourceRelation> listByTarget(String resourceId) {
        String sql = "select * from monitor_resource_relation where is_deleted = 0 and target = :target";
        Map<String, Object> paramMap = MapTools.simpleMap("target", resourceId);
        return this.list(ResourceRelation.class, sql, paramMap);
    }
}
