package com.bocloud.cms.repository;

import com.bocloud.cms.entity.Rule;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 告警规则
 *
 * @author fanzx
 * @date 2021/09/18
 */
@Repository
public class RuleRepository extends BasicGenericDao<Rule, Long> {

    public RuleRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据groupId查询Rule集合对象
     *
     * @param groupId 告警规则主键
     * @return 告警规则列表
     */
    public List<Rule> listByRuleGroupId(Long groupId) {
        String sql = "select * from monitor_rule where is_deleted = 0 and group_id = :groupId  ";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", groupId);
        List<Rule> rules = this.list(Rule.class, sql, paramMap);
        return ListTool.isEmpty(rules) ? null : rules;
    }

    /**
     * 根据ruleGroupId"删除"表信息
     *
     * @param id ID
     * @return 是否成功
     */
    public boolean delete(Long id) {
        String sql = "update monitor_rule set is_deleted = 1, gmt_modify = :gmtModify  where group_id = :groupId";
        Map<String, Object> paramMap = MapTools.simpleMap("groupId", id);
        paramMap.put("gmtModify", new Date());
        return this.execute(sql, paramMap) > 0;
    }
}
