package com.bocloud.cms.intf;

import com.bocloud.cms.entity.Alarm;
import com.megatron.common.model.*;

import java.util.Date;
import java.util.List;

/**
 * 告警服务service接口
 *
 * @author wangyu
 * @version 1.0
 * @since 2017年7月13日
 */
public interface AlarmService {

    /**
     * 告警查询
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 告警查询
     */
    GridBean<Alarm> list(Pager pager, RequestContext context);

    /**
     * 添加告警信息
     *
     * @param alarm 告警
     */
    void create(Alarm alarm);

    /**
     * 批量添加告警信息
     *
     * @param alarms 告警集合
     */
    void create(List<Alarm> alarms);

    /**
     * 获取详细信息
     *
     * @param id 告警ID
     * @return 告警详细信息
     */
    GeneralResult<Alarm> detail(Long id);

    /**
     * 查询未解决并且需要发送通知的告警
     *
     * @return 未解决并且需要发送通知的告警集合
     */
    List<Alarm> unSolvesForSendNotice();

    /**
     * 告警确认
     *
     * @param ids     告警ID
     * @param context 当前用户上下文 操作者
     * @return 操作结果
     */
    GeneralResult<Void> confirm(List<Long> ids, RequestContext context);

    /**
     * 告警解决
     *
     * @param id         告警ID
     * @param context    当前用户上下文
     * @param dealResult 处理结果
     */
    void solve(Long id, RequestContext context, String dealResult);

    /**
     * 告警解决(批量)
     *
     * @param ids        告警ID集合
     * @param context    当前用户上下文
     * @param dealResult 处理结果
     */
    void solve(List<Long> ids, RequestContext context, String dealResult);

    /**
     * 按云主机解决告警
     *
     * @param id           告警ID
     * @param userId       用户ID
     * @param resourceType 资源类型
     * @param dealResult   处理结果
     */
    void solveByCloudServer(Long id, String resourceType, Long userId, String dealResult);

    /**
     * 按平台解决告警
     *
     * @param id         告警ID
     * @param userId     用户ID
     * @param dealResult 处理结果
     */
    void solveByVendor(Long id, Long userId, String dealResult);

    /**
     * 查询资源告警主记录
     *
     * @param name     告警名称
     * @param level    告警级别
     * @param ruleId   规格ID
     * @param target   告警目标
     * @param tenantId 租户ID
     * @param trigger  触发时间
     * @return 告警
     */
    Alarm queryAlarmMain(String name, String level, Long ruleId, String target, Long tenantId, Date trigger);

    /**
     * 统计接口
     *
     * @param params  参数集合
     * @param context 当前用户上下文
     * @return 统计信息
     */
    GeneralResult<?> stats(List<Param> params, RequestContext context);

    /**
     * 告警统计饼状图
     *
     * @param context    当前用户上下文
     * @param start      开始时间
     * @param end        结束时间
     * @param vendorId   云平台ID
     * @param target     告警目标
     * @param type       类型
     * @param vendorType 云平台类型
     * @param time       时间
     * @return 告警统计信息
     */
    GeneralResult<?> alarmLevel(RequestContext context, Long start, Long end, Long vendorId, Object target, String type,
                                String vendorType, String time);

    /**
     * 告警统计柱状堆积图
     *
     * @param context    当前用户上下文
     * @param type       类型
     * @param vendorId   云平台ID
     * @param target     告警目标
     * @param start      开始时间
     * @param end        结束时间
     * @param vendorType 云平台类型
     * @return 告警统计信息
     */
    GeneralResult<?> alarmStats(RequestContext context, String type, Long vendorId, Object target, Long start, Long end,
                                String vendorType);

    /**
     * 平台告警柱状图
     *
     * @param context    当前用户上下文
     * @param start      开始时间
     * @param end        结束时间
     * @param vendorType 云平台类型
     * @return 平台告警柱状图
     */
    GeneralResult<?> vendorAlarmStats(RequestContext context, Long start, Long end, String vendorType);

}
