package com.bocloud.cms.intf;

import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * prometheus监控数据查询
 *
 * @author zhangyf
 * @since 2018-12-27
 */
public interface PrometheusService {

    /**
     * query查询方式请求
     *
     * @param expr 查询条件
     * @return 监控数据
     */
    Double simpleQuery(String expr);

    /**
     * query查询方式请求
     *
     * @param expr 查询条件
     * @param time 当前请求时间点的时间戳
     * @return 监控数据
     */
    Double simpleQuery(String expr, String time);

    /**
     * query查询方式请求
     *
     * @param expr   查询表达式
     * @param params 参数
     * @return 监控数据
     */
    Double simpleQuery(String expr, Map<String, String> params);

    /**
     * query查询方式请求
     *
     * @param expr   查询表达式
     * @param params 参数
     * @return 监控数据
     */
    Double simpleQuery(String expr, List<LabelParam> params);

    /**
     * query查询方式请求
     *
     * @param expr 查询条件
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr);

    /**
     * query查询方式请求
     *
     * @param expr   查询条件
     * @param params 参数
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr, Map<String, String> params);

    /**
     * query查询方式请求
     *
     * @param expr   查询条件
     * @param params 参数
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr, List<LabelParam> params);

    /**
     * query查询方式请求
     *
     * @param expr 查询条件
     * @param time 当前请求时间点的时间戳
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr, String time);

    /**
     * query查询方式请求
     *
     * @param expr   查询条件
     * @param time   当前请求时间点的时间戳
     * @param params 参数
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr, Map<String, String> params, String time);

    /**
     * query查询方式请求
     *
     * @param expr   查询条件
     * @param time   当前请求时间点的时间戳
     * @param params 参数
     * @return 监控数据
     */
    List<PrometheusResult> query(String expr, List<LabelParam> params, String time);

    /**
     * query_range查询方式请求
     *
     * @param start 过去某个时间点的时间戳
     * @param end   当前请求时间点的时间戳
     * @param step  获取时间与时间之间数据的步长
     * @param expr  查询条件
     * @return 监控数据
     */
    List<PrometheusResult> queryRange(String expr, String start, String end, String step);

    /**
     * query_range查询方式请求
     *
     * @param expr   查询条件
     * @param params 参数
     * @param start  过去某个时间点的时间戳
     * @param end    当前请求时间点的时间戳
     * @param step   获取时间与时间之间数据的步长
     * @return 监控数据
     */
    List<PrometheusResult> queryRange(String expr, Map<String, String> params, String start, String end, String step);

    /**
     * query_range查询方式请求
     *
     * @param expr   查询条件
     * @param params 参数
     * @param start  过去某个时间点的时间戳
     * @param end    当前请求时间点的时间戳
     * @param step   获取时间与时间之间数据的步长
     * @return 监控数据
     */
    List<PrometheusResult> queryRange(String expr, List<LabelParam> params, String start, String end, String step);

    /**
     * 过滤集合
     *
     * @param expr   表达示
     * @param filter 要过滤元素
     * @return 过滤后的Label值集合
     */
    Set<String> filter(String expr, String filter);

    /**
     * 重新加载prometheus配置信息
     */
    void reload();

}