package com.bocloud.cms.intf;

import com.bocloud.cms.entity.RuleGroup;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;

import java.util.List;

/**
 * @author fanzx
 * @date 2021/09/18
 */
public interface RuleGroupService {

    /**
     * 添加告警策略功能
     *
     * @param ruleGroup 告警策略
     * @param context   当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> create(RuleGroup ruleGroup, RequestContext context);

    /**
     * 告警策略分页列表展示
     *
     * @param pager   分页
     * @param context 当前用户上下文
     * @return 告警策略集合
     */
    GeneralResult<?> list(Pager pager, RequestContext context);

    /**
     * 删除告警策略（单条）
     *
     * @param id      告警策略ID
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> remove(Long id, RequestContext context);

    /**
     * 批量删除阈值
     *
     * @param ids    告警策略ID集合
     * @param userId 用户ID
     * @return 操作结果
     */
    GeneralResult<?> batchRemove(List<Long> ids, Long userId);

    /**
     * 修改告警策略信息
     *
     * @param id        告警策略ID
     * @param ruleGroup 告警策略
     * @param context   当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> modify(Long id, RuleGroup ruleGroup, RequestContext context);

    /**
     * 告警策略详情
     *
     * @param id 告警策略ID
     * @return 告警策略详情
     */
    RuleGroup detail(Long id);

    /**
     * 开启/关闭   告警策略信息
     *
     * @param groupId 告警策略ID
     * @param enable  是否开启
     * @param context 当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> enabled(Long groupId, Boolean enable, RequestContext context);

    /**
     * 云平台关联告警策略功能
     *
     * @param ruleGroupIds 告警策略ID集合
     * @param resourceId   资源ID
     * @param resourceType 资源类型
     * @param context      当前用户上下文
     * @return 操作结果
     */
    GeneralResult<?> binding(List<Long> ruleGroupIds, String resourceId, String resourceType, RequestContext context);

    /**
     * 云平台关联告警策略功能
     * 通过resourceId,查询到已经关联的告警策略列表
     *
     * @param resourceId 资源ID
     * @param context    当前用户上下文
     * @return 已经关联的告警策略列表
     */
    GeneralResult<?> bindingList(String resourceId, RequestContext context);

}
