package com.bocloud.cms.intf;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.ims.entity.CloudVendor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;

import java.util.List;

/**
 * 平台监控
 *
 * @author 王文凯
 * @date 2019年1月28日 上午10:34:38
 */
public interface VendorService {
    /**
     * 开启监控(目前仅支持单个操作，不支持批量)
     *
     * @param context 当前用户上下文
     * @param idsList 云平台ID集合
     * @return 操作结果
     */
    GeneralResult<?> open(RequestContext context, List<Long> idsList);

    /**
     * 关闭监控(目前仅支持单个操作，不支持批量)
     *
     * @param context 当前用户上下文
     * @param idsList 云平台ID集合
     * @return 操作结果
     */
    GeneralResult<?> close(RequestContext context, List<Long> idsList);

    /**
     * 配置xsky prometheus信息
     *
     * @param id         云平台ID
     * @param jsonObject prometheus配置信息
     */
    void updateXskyMonitoringInfo(Long id, JSONObject jsonObject);


    /**
     * 关闭xsky监控
     *
     * @param vendor 云平台
     */
    void closeXsky(CloudVendor vendor);

    /**
     * 关闭Openstack、Kvm监控
     *
     * @param vendor 云平台
     */
    void closeOpenstackOrKvm(CloudVendor vendor);


}
