package com.bocloud.cms.service;

import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.intf.AnalysisRuleService;
import com.bocloud.cms.repository.AnalysisRuleRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 监控分析Service接口
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.5.0
 */
@Service
@Slf4j
public class AnalysisRuleServiceImpl implements AnalysisRuleService {

    @Autowired
    private AnalysisRuleRepository analysisRuleRepository;
    @Autowired
    private LockFactory lockFactory;


    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GridBean gridBean;
        try {
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(Maps.newHashMap()));
            pager.getSorter().put("gmtCreate", Common.ONE);
            int total = this.analysisRuleRepository.count(pager.getParams());
            List<MonitorAnalysisRule> list = this.analysisRuleRepository.list(pager);
            gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("Query threshold list fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult create(MonitorAnalysisRule analysisRule, RequestContext context) {
        try {
            analysisRule.setCreatorId(context.getTarget());
            analysisRule.setGmtCreate(new Date());
            analysisRule.setDeleted(false);
            boolean exist = analysisRuleRepository.nameExist(null, analysisRule.getName());
            Assert.isTrue(!exist, " 名称已存在");
            analysisRuleRepository.save(analysisRule);
            return new GeneralResult<>(true, "新增监控分析规则成功");
        } catch (Exception e) {
            log.error("Create threshold error!", e);
            return new GeneralResult<>(false, "新增监控分析规则失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult modify(MonitorAnalysisRule analysisRule, RequestContext context) {
        String path = MonitorAnalysisRule.class.getSimpleName() + "_" + analysisRule.getId();
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), " 请求超时");
            MonitorAnalysisRule object = this.analysisRuleRepository.query(analysisRule.getId());
            Assert.notNull(object, " 数据不存在");
            BeanUtils.copyProperties(analysisRule, object);
            object.setGmtModify(new Date());
            object.setMenderId(context.getTarget());
            object.setDeleted(false);
            boolean exist = analysisRuleRepository.nameExist(analysisRule.getId(), analysisRule.getName());
            Assert.isTrue(!exist, " 名称已存在");
            this.analysisRuleRepository.update(object);
            return new GeneralResult<>(true, object, "监控分析规则修改成功");
        } catch (Exception e) {
            log.error("Modify monitor analysis rule error:", e);
            return new GeneralResult<>(false, "监控分析规则修改失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult remove(Long id, Long contextId) {
        String path = MonitorAnalysisRule.class.getSimpleName() + "_" + id;
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            MonitorAnalysisRule object = this.analysisRuleRepository.query(id);
            Assert.notNull(object, " 数据不存在");
            object.setDeleted(true);
            object.setMenderId(contextId);
            analysisRuleRepository.update(object);
            return new GeneralResult<>(true, "监控分析规则删除成功");
        } catch (Exception e) {
            log.error("Remove monitor analysis rule error:", e);
            return new GeneralResult<>(false, "监控分析规则删除失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult detail(Long id) {
        try {
            MonitorAnalysisRule analysisRule = this.analysisRuleRepository.query(id);
            Assert.notNull(analysisRule, " 数据不存在");
            return new GeneralResult<>(true, analysisRule, "监控分析规则详情展示成功");
        } catch (Exception e) {
            log.error("threshold detail error!", e);
            return new GeneralResult<>(false, "监控分析规则详情展示失败" + e.getMessage());
        }
    }


}
