package com.bocloud.cms.service;

import com.bocloud.cms.entity.AnalysisSnapshot;
import com.bocloud.cms.entity.MonitorAnalysisRule;
import com.bocloud.cms.intf.AnalysisSnapshotService;
import com.bocloud.cms.repository.AnalysisRuleRepository;
import com.bocloud.cms.repository.AnalysisSnapshotRepository;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.ims.entity.CloudVendor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 监控分析快照
 *
 * @author zhangyf
 * @since 2019-08-15
 */
@Service
@Slf4j
public class AnalysisSnapshotServiceImpl implements AnalysisSnapshotService {

    @Autowired
    private AnalysisSnapshotRepository analysisSnapshotRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private AnalysisRuleRepository analysisRuleRepository;
    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;

    @Override
    public GeneralResult<GridBean<AnalysisSnapshot>> list(Pager pager, RequestContext context) {
        Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()).put("gmtCreate", Common.ONE);
        int total = this.analysisSnapshotRepository.count(pager.getParams());
        List<AnalysisSnapshot> list = this.analysisSnapshotRepository.list(pager);
        GridBean<AnalysisSnapshot> gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
        return new GeneralResult<>(true, gridBean, "查询成功");
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = AnalysisSnapshot.class.getSimpleName() + "_" + id;
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), " 请求超时");
            AnalysisSnapshot object = this.analysisSnapshotRepository.query(id);
            Assert.notNull(object, " 数据不存在");
            object.setDeleted(true);
            analysisSnapshotRepository.update(object);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Remove snapshot error:", e);
            return new GeneralResult<>(false, "删除失败" + e.getMessage());
        }
    }

    @Override
    public GeneralResult<AnalysisSnapshot> detail(Long id) {
        AnalysisSnapshot snapshot = this.analysisSnapshotRepository.query(id);
        Assert.notNull(snapshot, " 数据不存在");
        MonitorAnalysisRule analysisRule = analysisRuleRepository.query(snapshot.getRuleId());
        CloudVendor vendor = cmpCloudVendorService.detail(snapshot.getVendorId()).getData();
        snapshot.setType(analysisRule.getType());
        snapshot.setRuleName(analysisRule.getName());
        snapshot.setVendorName(vendor.getName());
        return new GeneralResult<>(true, snapshot, "详情展示成功");
    }

}
