package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wpj
 * @since 2022/10/11
 */
@Slf4j
@Service
public class ApsaraStackServiceImpl  implements MonitoringAble<CloudServer>, InitializingBean {
    @Autowired
    private CmpCloudServerService cmpCloudServerService;
    @Autowired
    private AlarmService alarmService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;



    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        params.put("instanceId", targetObject.getInstanceId());
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String uuid = String.valueOf(alert.getLabels().get("instanceId"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("instanceId", uuid), Sign.EQ));
        GeneralResult result = cmpCloudServerService.list(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        Assert.isTrue(list != null && !list.isEmpty(), "apsarastack ecs not found,uuid:" + uuid);
        return list.get(0);
    }

    @Override
    public void destroy(CloudServer object, String catalog) throws Exception {
        log.info("destroy apsarastack ecs  : " + object.getId());
        alarmService.solveByCloudServer(object.getId(), catalog, 1L, "系统解决");
    }


    @Override
    public void wrapAlarm(Alarm alarm, CloudServer object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        if (object.getName()!=null & object.getPrivateIps()!=null){
            alarm.setTargetName(object.getName() + "(" + object.getManagerIp() + ")");
        }else{
            String targetName = object.getManagerIp() == null ? object.getName() : object.getManagerIp();
            alarm.setTargetName(targetName);
        }
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_APSARASTACK_VM, this);
    }
}
