package com.bocloud.cms.service;

import com.bocloud.cms.intf.CephService;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * CephOsdService实现层
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
public class CephServiceImpl implements CephService {

    @Autowired
    private PrometheusService prometheusService;

    @Override
    public List<Map<String, Object>> stats() {
        List<PrometheusResult> metadataList = prometheusService.query("ceph_osd_metadata");
        Map<String, Map<String, Object>> map = new HashMap<>(16, 0.75F);
        Optional.ofNullable(metadataList).orElse(new ArrayList<>()).stream().forEach(result -> {
            String daemon = String.valueOf(result.getMetric().get("ceph_daemon"));
            String instance = String.valueOf(result.getMetric().get("instance"));
            map.put(instance + daemon, result.getMetric());
        });
        List<PrometheusResult> totalList = prometheusService.query("round(ceph_osd_stat_bytes/1024^3,0.01)");
        Optional.ofNullable(totalList).orElse(new ArrayList<>()).stream().forEach(result -> {
            String daemon = String.valueOf(result.getMetric().get("ceph_daemon"));
            String instance = String.valueOf(result.getMetric().get("instance"));
            map.get(instance + daemon).put("total", result.getValue()[1]);
        });

        List<PrometheusResult> usedList = prometheusService.query("round(ceph_osd_stat_bytes_used/1024^3,0.01)");
        Optional.ofNullable(usedList).orElse(new ArrayList<>()).stream().forEach(result -> {
            String daemon = String.valueOf(result.getMetric().get("ceph_daemon"));
            String instance = String.valueOf(result.getMetric().get("instance"));
            map.get(instance + daemon).put("used", result.getValue()[1]);
        });

        List<Map<String, Object>> result = new ArrayList<>();
        map.forEach((key, value) -> result.add(value));
        return result;
    }

}
