package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.DataStoreModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.CmpCloudVendorService;
import com.bocloud.cms.service.internal.vsphere.DataStoreService;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.entity.DataStore;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * vc数据存储
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
public class DataStoreServiceImpl implements com.bocloud.cms.intf.DataStoreService, InitializingBean {

    @Autowired
    private DataStoreService dataStoreService;
    @Autowired
    private CmpCloudVendorService cmpCloudVendorService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;


    @Override
    public GeneralResult list(Pager pager, RequestContext context) {
        GeneralResult result = dataStoreService.list(pager);
        JSONObject data = (JSONObject) result.getData();
        List<DataStoreModel> list = data.getJSONArray(Common.ROWS).toJavaList(DataStoreModel.class);

        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_VCENTER_DATASTORE.name(), targets, context);

        //查询关联关系
        for (DataStoreModel dataStore : list) {
            targets.add(dataStore.getId().toString());
            GeneralResult<CloudVendor> cloudVendor = cmpCloudVendorService.detail(dataStore.getVendorId());
            String address = JSONObject.parseObject(cloudVendor.getData().getAuthentication()).getString("address");
            dataStore.setAddress(address);

            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(dataStore.getId().toString()))
                    .findAny().isPresent();
            dataStore.setAlarmEnable(alarmEnable);
        }

        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<DataStore> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = dataStoreService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(DataStore.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(DataStore dataStore, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("dsname", dataStore.getName());
        params.put("vendor_id", String.valueOf(dataStore.getVendorId()));
        return params;
    }

    @Override
    public DataStore parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String name = String.valueOf(alert.getLabels().get("dsname"));
        String vendorId = String.valueOf(alert.getLabels().get("vendor_id"));
        List<Param> params = new ArrayList<>(1);
        params.add(new Param(MapTools.simpleMap("name", name), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("vendorId", vendorId), Sign.EQ));
        GeneralResult result = dataStoreService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<DataStore> list = data.getJSONArray(Common.ROWS).toJavaList(DataStore.class);
        Assert.isTrue(list != null && !list.isEmpty(), "DataStore not found,vendorId:" + vendorId + " name:" + name);
        return list.get(0);
    }

    @Override
    public void destroy(DataStore object, String catalog) throws Exception {
        //TODO
    }

    @Override
    public void wrapAlarm(Alarm alarm, DataStore object) {
        alarm.setTarget(object.getId());
        alarm.setTargetName(object.getName());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, DataStore object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_VCENTER_DATASTORE, this);
    }
}

