package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.HcsoElbListenerService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.LbListenerModel;
import com.bocloud.cms.model.LoadBalancerModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.HcsoService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hya
 * @date 2022/5/12 16:45
 */
@Service
public class HcsoElbListenerServiceImpl implements HcsoElbListenerService, InitializingBean {

    @Autowired
    private HcsoService hcsoService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    /**
     * 查询负载均衡监听器列表 暂时先不做
     *
     * @param pager 分页
     * @param context 当前用户上下文
     * @param request
     * @return
     */
    @Override
    public GeneralResult listOfLbListener(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = hcsoService.listOfLbListener(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<LbListenerModel> list = data.getJSONArray(Common.ROWS).toJavaList(LbListenerModel.class);
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HCSO_ELB_LISTENER.name(), targets, context);
        for (LbListenerModel lbListenerModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(lbListenerModel.getId().toString()))
                    .findAny().isPresent();
            lbListenerModel.setAlarmEnable(alarmEnable);

            Param param = new Param(MapTools.simpleMap("lbUuid", lbListenerModel.getLbUuid()), Sign.EQ);
            List<Param> params = Collections.singletonList(param);
            GeneralResult loadBalancerResult = hcsoService.listOfLoadBalancer(new Pager(1, Integer.MAX_VALUE, params, null), null);
            JSONObject lbData = (JSONObject) loadBalancerResult.getData();
            List<LoadBalancerModel> lbList = lbData.getJSONArray(Common.ROWS).toJavaList(LoadBalancerModel.class);
            lbListenerModel.setLbName(lbList.get(0).getName());
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<LbListenerModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = hcsoService.listOfLbListener(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(LbListenerModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(LbListenerModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        // 监听器ID
        params.put("listener_id", targetObject.getListenerUuid());
        params.put("vendor_id", targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public LbListenerModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String listenerId = String.valueOf(alert.getLabels().get("listener_id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("listenerUuid", listenerId), Sign.EQ));
        GeneralResult result = hcsoService.listOfLbListener(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<LbListenerModel> list = data.getJSONArray(Common.ROWS).toJavaList(LbListenerModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "huawei elb listener not found,listenerUuid:" + listenerId);
        return list.get(0);
    }

    @Override
    public void destroy(LbListenerModel object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, LbListenerModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getName() + "(" + object.getListenerUuid() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, LbListenerModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HCSO_ELB_LISTENER, this);
    }
}