package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.HcsoElbService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.LbListenerModel;
import com.bocloud.cms.model.LoadBalancerModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.HcsoService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hya
 * @date 2022/5/12 16:45
 */
@Service
@Slf4j
public class HcsoElbServiceImpl implements HcsoElbService, InitializingBean {

    @Autowired
    private HcsoService hcsoService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;

    @Override
    public GeneralResult listOfLoadBalancer(Pager pager, RequestContext context, HttpServletRequest request) {
        // 查询负载均衡器列表
        GeneralResult result = hcsoService.listOfLoadBalancer(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<LoadBalancerModel> list = data.getJSONArray(Common.ROWS).toJavaList(LoadBalancerModel.class);
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        // 获取告警状态
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HCSO_ELB.name(), targets, context);
        // 负载均衡器 是否屏蔽告警
        for (LoadBalancerModel loadBalancerModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(loadBalancerModel.getId().toString()))
                    .findAny().isPresent();
            loadBalancerModel.setAlarmEnable(alarmEnable);

            // 如果listenerNum不为空 则代表该负载均衡下有监听器
            if (loadBalancerModel.getListenerNum() != 0) {
                Param param = new Param(MapTools.simpleMap("lbId", loadBalancerModel.getId()), Sign.IN);
                List<Param> params = Collections.singletonList(param);
                GeneralResult GeneralResult = hcsoService.listOfLbListener(new Pager(1, Integer.MAX_VALUE, params, null), null);
                JSONObject listenerData = (JSONObject) GeneralResult.getData();
                List<LbListenerModel> listenerList = listenerData.getJSONArray(Common.ROWS).toJavaList(LbListenerModel.class);
                List<String> listenerTargets = listenerList.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
                // 获取告警状态
                List<ResourceAlarmStatus> listenerAlarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HCSO_ELB.name(), listenerTargets, context);
                loadBalancerModel.setLbListenerModels(listenerList);
                for (LbListenerModel lbListenerModel : listenerList) {
                    //告警启用状态，默认开启
                    boolean listenerAlarmEnable = !Optional.ofNullable(listenerAlarmStatuses).orElse(new ArrayList<>()).stream()
                            .filter(a -> a.getResourceId().equals(lbListenerModel.getId().toString()))
                            .findAny().isPresent();
                    lbListenerModel.setAlarmEnable(listenerAlarmEnable);
                }
            }
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<LoadBalancerModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = hcsoService.listOfLoadBalancer(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(LoadBalancerModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(LoadBalancerModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        // 负载均衡器ID
        params.put("loadbalancer_id", targetObject.getLbUuid());
        params.put("vendor_id", targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public LoadBalancerModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String lbUuid = String.valueOf(alert.getLabels().get("loadbalancer_id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("lbUuid", lbUuid), Sign.EQ));
        GeneralResult result = hcsoService.listOfLoadBalancer(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<LoadBalancerModel> list = data.getJSONArray(Common.ROWS).toJavaList(LoadBalancerModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "huawei LoadBalancer not found,lbId:" + lbUuid);
        return list.get(0);
    }

    @Override
    public void destroy(LoadBalancerModel object, String catalog) throws Exception {
        log.info("destroy huawei loadBalancer  : " + object.getId());
        alarmService.solveByCloudServer(object.getId(), catalog, 1L, "系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, LoadBalancerModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getName();
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, LoadBalancerModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HCSO_ELB, this);
    }
}