package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.*;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.HcsoNatService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.NatGatewayModel;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.cms.service.internal.HcsoService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author hya
 * @date 2022/5/12 16:44
 */
@Service
@Slf4j
public class HcsoNatServiceImpl implements HcsoNatService, InitializingBean {

    @Autowired
    private HcsoService hcsoService;
    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;
    @Autowired
    private AlarmService alarmService;

    @Override
    public GeneralResult listOfNat(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult result = hcsoService.listOfNat(pager, request.getHeader(Common.TOKEN));
        JSONObject data = (JSONObject) result.getData();
        List<NatGatewayModel> list = data.getJSONArray(Common.ROWS).toJavaList(NatGatewayModel.class);
        List<String> targets = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HCSO_NAT.name(), targets, context);
        for (NatGatewayModel natGatewayModel : list) {
            //告警启用状态，默认开启
            boolean alarmEnable = !Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .filter(a -> a.getResourceId().equals(natGatewayModel.getId().toString()))
                    .findAny().isPresent();
            natGatewayModel.setAlarmEnable(alarmEnable);
        }
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public List<NatGatewayModel> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetIds).substring(1, JSONObject.toJSONString(targetIds).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = hcsoService.listOfNat(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(NatGatewayModel.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(NatGatewayModel targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>();
        // 网关ID
        params.put("nat_id", targetObject.getUuid());
        params.put("vendor_id", targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public NatGatewayModel parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String natId = String.valueOf(alert.getLabels().get("nat_id"));
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("uuid", natId), Sign.EQ));
        GeneralResult result = hcsoService.listOfNat(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<NatGatewayModel> list = data.getJSONArray(Common.ROWS).toJavaList(NatGatewayModel.class);
        Assert.isTrue(list != null && !list.isEmpty(), "huawei nat not found,natId:" + natId);
        return list.get(0);
    }

    @Override
    public void destroy(NatGatewayModel object, String catalog) throws Exception {
        log.info("destroy huawei NatGateway  : " + object.getId());
        alarmService.solveByCloudServer(object.getId(), catalog, 1L, "系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, NatGatewayModel object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getName();
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, NatGatewayModel object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_HCSO_NAT, this);
    }
}