package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.ManageOneBmsService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.BareMetalInfo;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

@Service
@Slf4j
public class ManageOneBmsServiceImpl implements ManageOneBmsService, InitializingBean {

    @Autowired
    private CmpCloudServerService cmpCloudServerService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public List<BareMetalInfo> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.listManagerBms(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(BareMetalInfo.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(BareMetalInfo targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("instanceId", targetObject.getInstanceId());
        params.put("vendorId",targetObject.getVendorId().toString());
        return params;
    }

    @Override
    public BareMetalInfo parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String vm = String.valueOf(alert.getLabels().get("instanceId"));
        // 通过vmName去查找对应的云主机信息
        List<Param> params = Collections.singletonList(new Param(MapTools.simpleMap("clusterUuid", vm), Sign.EQ));
        GeneralResult result = cmpCloudServerService.listManagerBms(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<BareMetalInfo> list = data.getJSONArray(Common.ROWS).toJavaList(BareMetalInfo.class);
        Assert.isTrue(list != null && !list.isEmpty(), "smartXvm not found,vm:" + vm);
        return list.get(0);
    }

    @Override
    public void destroy(BareMetalInfo object, String catalog) throws Exception {
        log.info("destroy Manage bms : " + object.getId());
        alarmService.solveByCloudServer(object.getId(),catalog,1L,"系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, BareMetalInfo object) {
        alarm.setTarget(object.getId());
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        String targetName = object.getIpAddress() == null ? object.getName() : object.getName() + "(" + object.getIpAddress() + ")";
        alarm.setTargetName(targetName);
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, BareMetalInfo object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());

    }

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_MANAGEONE_BMS, this);

    }
}
