package com.bocloud.cms.service;

import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.MonitoringAble;

import java.util.HashMap;
import java.util.Map;

/**
 * 监控注册中心
 *
 * @author zhangyf
 * @since 2020-04-07
 */
public class MonitoringAbleRegistry {

    private MonitoringAbleRegistry(){

    }

    private static final Map<String, MonitoringAble> MONITORING_ABLES = new HashMap<>(16, 0.75F);

    public static void register(MonitorResourceType monitorResourceType, MonitoringAble monitoringAble) {
        synchronized (MONITORING_ABLES) {
            if (MONITORING_ABLES.containsKey(monitorResourceType.name())) {
                throw new IllegalArgumentException("MonitoringAble already registered that provides resource type: " + monitorResourceType.name());
            }
            MONITORING_ABLES.put(monitorResourceType.name(), monitoringAble);
        }
    }

    /**
     * 获取监控对象告警处理器实例
     *
     * @param monitorResourceType
     * @return
     */
    public static MonitoringAble getInstance(MonitorResourceType monitorResourceType) {
        return MonitoringAbleRegistry.MONITORING_ABLES.get(monitorResourceType.name());
    }

}
