package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.PrometheusConfig;
import com.bocloud.cms.common.utils.Constant;
import com.bocloud.cms.common.utils.PrometheusUtils;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.google.common.collect.Maps;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;
import com.megatron.common.model.*;

import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

/**
 * prometheusService层
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Component
@Slf4j
public class PrometheusServiceImpl implements PrometheusService {


    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("reload-prometheus-pool-%d").build();
    private ExecutorService reloadPrometheusPool = new ThreadPoolExecutor(1, 1, 0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(1), nameThreadFactory, new ThreadPoolExecutor.DiscardOldestPolicy());

    @Autowired
    private PrometheusConfig prometheusConfig;

    @Override
    public Double simpleQuery(String expr) {
        return simpleQuery(expr, new HashMap<>(16, 0.75F));
    }

    @Override
    public Double simpleQuery(String expr,String time) {
        List<PrometheusResult> prometheusResult = query(expr, time);
        if (prometheusResult != null && !prometheusResult.isEmpty()) {
            if("NaN".equalsIgnoreCase(String.valueOf(prometheusResult.get(0).getValue()[1]))){
                return 0D;
            }
            return Double.valueOf(String.valueOf(prometheusResult.get(0).getValue()[1]));
        }
        return null;
    }

    @Override
    public Double simpleQuery(String expr, Map<String, String> params) {
        List<PrometheusResult> prometheusResult = query(expr, params);
        if (prometheusResult != null && !prometheusResult.isEmpty()) {
            if("NaN".equalsIgnoreCase(String.valueOf(prometheusResult.get(0).getValue()[1]))){
                return 0D;
            }
            return Double.valueOf(String.valueOf(prometheusResult.get(0).getValue()[1]));
        }
        return null;
    }

    @Override
    public Double simpleQuery(String expr, List<LabelParam> params) {
        List<PrometheusResult> prometheusResult = query(expr, params);
        if (prometheusResult != null && !prometheusResult.isEmpty()) {
            if("NaN".equalsIgnoreCase(String.valueOf(prometheusResult.get(0).getValue()[1]))){
                return 0D;
            }
            return Double.valueOf(String.valueOf(prometheusResult.get(0).getValue()[1]));
        }
        return null;
    }

    @Override
    public List<PrometheusResult> query(String expr) {
        return query(expr, new HashMap<>(16, 0.75F), null);
    }

    @Override
    public List<PrometheusResult> query(String expr, Map<String, String> params) {
        return query(expr, params, null);
    }

    @Override
    public List<PrometheusResult> query(String expr, List<LabelParam> params) {
        return query(expr, params, null);
    }

    @Override
    public List<PrometheusResult> query(String expr, String time) {
        return query(expr, new HashMap<>(16, 0.75F), time);
    }

    @Override
    public List<PrometheusResult> query(String expr, Map<String, String> params, String time) {
        String url = getRequestUrl(Constant.Monitor.QUERY_URL);
        Map<String, Object> paramsMap = new HashMap<>(16, 0.75F);
        paramsMap.put(Constant.Monitor.QUERY, PrometheusUtils.putParams(expr, params));
        paramsMap.put(Constant.Monitor.TIME_TIMESTAMP, time);
        return sendRequest(url, paramsMap);
    }

    @Override
    public List<PrometheusResult> query(String expr, List<LabelParam> params, String time) {
        String url = getRequestUrl(Constant.Monitor.QUERY_URL);
        Map<String, Object> paramsMap = new HashMap<>(16, 0.75F);
        paramsMap.put(Constant.Monitor.QUERY, PrometheusUtils.putParams(expr, params));
        paramsMap.put(Constant.Monitor.TIME_TIMESTAMP, time);
        return sendRequest(url, paramsMap);
    }

    @Override
    public List<PrometheusResult> queryRange(String expr, String start, String end, String step) {
        return queryRange(expr, new HashMap<>(16, 0.75F), start, end, step);
    }

    @Override
    public List<PrometheusResult> queryRange(String expr, Map<String, String> params, String start, String end, String step) {
        Double timeDifference = Double.parseDouble(end) - Double.parseDouble(start);
        if (timeDifference > Long.parseLong(step) * 10000) {
            step = String.valueOf((long) Math.ceil(timeDifference / 10000));
        }
        String url = getRequestUrl(Constant.Monitor.QUERY_RANGE_URL);
        Map<String, Object> paramsMap = new HashMap<>(16, 0.75F);
        paramsMap.put(Constant.Monitor.QUERY, PrometheusUtils.putParams(expr, params));
        paramsMap.put(Constant.Monitor.START_TIMESTAMP, start);
        paramsMap.put(Constant.Monitor.END_TIMESTAMP, end);
        paramsMap.put(Constant.Monitor.STEP, step);
        return sendRequest(url, paramsMap);
    }

    @Override
    public List<PrometheusResult> queryRange(String expr, List<LabelParam> params, String start, String end, String step) {
        String url = getRequestUrl(Constant.Monitor.QUERY_RANGE_URL);
        Map<String, Object> paramsMap = new HashMap<>(16, 0.75F);
        paramsMap.put(Constant.Monitor.QUERY, PrometheusUtils.putParams(expr, params));
        paramsMap.put(Constant.Monitor.START_TIMESTAMP, start);
        paramsMap.put(Constant.Monitor.END_TIMESTAMP, end);
        paramsMap.put(Constant.Monitor.STEP, step);
        return sendRequest(url, paramsMap);
    }

    /**
     * 请求监控服务获取监控数据
     *
     * @param url    请求路径
     * @param params 参数
     * @return 监控数据
     */
    private List<PrometheusResult> sendRequest(String url, Map<String, Object> params) {
        HttpClient httpClient = new HttpClient(prometheusConfig.getTimeout());
        Result result = httpClient.get(null, params, url);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        String response = result.getMessage();
        JSONObject monitorResult = JSONObject.parseObject(response);
        if ("success".equals(monitorResult.getString("status"))) {
            return monitorResult.getJSONObject("data").getJSONArray("result").toJavaList(PrometheusResult.class);
        } else {
            throw new RuntimeException(monitorResult.getString("error"));
        }
    }

    /**
     * 获取请求路径
     *
     * @param endpoint 请求类型
     * @return 请求路径
     */
    private String getRequestUrl(String endpoint) {
        return prometheusConfig.getUrl() + Constant.Monitor.API_VERSION_URL + endpoint;
    }


    @Override
    public Set<String> filter(String expr, String filter) {
        List<PrometheusResult> prometheusResults = this.query(expr);
        Set<String> result = new HashSet<>();
        for (PrometheusResult prometheusResult : prometheusResults) {
            result.add(String.valueOf(prometheusResult.getMetric().get(filter)));
        }
        return result;
    }

    @Override
    public void reload() {
        reloadPrometheusPool.execute(() -> {
            try {
                Thread.sleep(5000L);
            } catch (InterruptedException e) {
                log.error(e.getMessage(), e);
                Thread.currentThread().interrupt();
            }
            String[] addresses = Optional.ofNullable(prometheusConfig.getNodeUrls()).orElse("").split(",");
            Arrays.stream(Optional.ofNullable(addresses).orElse(new String[]{})).forEach(url ->
                    new HttpClient().post(Maps.newHashMap(), Maps.newHashMap(), url + Constant.Monitor.RELOAD_URL));
        });
    }

}