package com.bocloud.cms.service;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.intf.PrometheusRuleService;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * 开启关闭告警状态
 * @author zhangyf
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class ResourceAlarmStatusServiceImpl implements ResourceAlarmStatusService {


    @Autowired
    private ResourceAlarmStatusRepository alarmStatusRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PrometheusRuleService prometheusRuleService;

    /**
     * 屏蔽告警
     *
     * @param context 当前用户上下文
     * @return
     */
    @Override
    public GeneralResult remove(ResourceAlarmStatus alarmStatus, RequestContext context) {
        String path = ResourceAlarmStatus.class.getSimpleName() + "_" + alarmStatus.getResourceId();
        try (AutoCloseLock acLock = lockFactory.getACLock(path)) {
            Assert.notNull(alarmStatus,"未找到指定的信息!");
            Assert.isTrue(acLock.acquire(10, TimeUnit.SECONDS), "请求超时");
            alarmStatus.setTenantId(context.getTenant());
            alarmStatus.setGmtCreate(new Date());
            alarmStatus.setEnable(false);
            this.alarmStatusRepository.save(alarmStatus);
            return new GeneralResult<>(true, "屏蔽告警成功");
        } catch (Exception e) {
            log.error("Remove alarmStatus error:", e);
            return new GeneralResult<>(false, "屏蔽告警失败：" + e.getMessage());
        }
    }

    /**
     * 开启告警  true开启
     *
     * @param alarmStatus
     * @param context 当前用户上下文
     * @return
     */
    @Override
    public GeneralResult create(ResourceAlarmStatus alarmStatus, RequestContext context) {

        try {
            Assert.notNull(alarmStatus,"未找到指定的信息!");
            this.alarmStatusRepository.delete(alarmStatus.getResourceId(),alarmStatus.getResourceType(),context);
            return new GeneralResult<>(true, "关联告警成功");
        } catch (Exception e) {
            log.error("alarmStatus relation error:", e);
            return new GeneralResult<>(false, "关联告警失败：" + e.getMessage());
        }
    }


    @Override
    public boolean alarmEnable(String resourceType, String target, Long tenantId) {
        //验证管理端是否屏蔽告警
        boolean alarmEnable = alarmStatusRepository.alarmEnable(resourceType, target, 0L);
        //管理端屏蔽告警，则自服务端也屏蔽告警
        if (0L == tenantId || !alarmEnable) {
            return alarmEnable;
        }
        return alarmStatusRepository.alarmEnable(resourceType, target, tenantId);
    }

}
