package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.common.config.StorageConfig;
import com.bocloud.cms.common.utils.SshdExecutor;
import com.bocloud.cms.intf.TelegrafService;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;
import com.megatron.framework.registry.RegistryClient;
import com.megatron.framework.registry.ZooKeeperClient;
import com.moandjiezana.toml.Toml;
import com.moandjiezana.toml.TomlWriter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * PrometheusTelegraf
 * @author liuyuanyuan
 * @version 1.0
 * @date 2022/2/21 18:01
 * @version 5.6.0
 */
@Service
@Slf4j
public class TelegrafServiceImpl implements TelegrafService {


    @Value("${deploy.mode.docker:false}")
    private boolean isDocker;
    @Autowired
    StorageConfig storageConfig;
    @Autowired
    private ZooKeeperClient zooKeeperClient;
    @Autowired
    private RegistryClient registryClient;

    @Override
    public void createConfigFile(CloudVendor vendor) throws Exception {
        try (InputStream inputStream = this.getClass().getResourceAsStream("/template/telegraf/vsphere.conf")) {
            // 每个vmware平台对应一个telegraf配置文件，以平台uuid为文件名
            Toml toml = new Toml().read(inputStream);
            List<Map<String, Object>> list = toml.getList("inputs.vsphere");
            String username = JSONObject.parseObject(vendor.getAuthentication()).getString("account");
            String password = new DESEncryptor().decrypt(JSONObject.parseObject(vendor.getAuthentication()).getString("password"), "BocloudCMP");
            String adress = JSONObject.parseObject(vendor.getAuthentication()).getString("address");
            list.forEach(map -> {
                map.put("username", username);
                map.put("password", password);
                map.put("vcenters", Arrays.asList(adress));
            });
            TomlWriter tomlWriter = new TomlWriter.Builder().build();
            Map<String, Object> inputs = new HashMap<>(16, 0.75F);
            Map<String, List<Map<String, Object>>> vsphere = new HashMap<>(16, 0.75F);
            vsphere.put("vsphere", list);
            inputs.put("inputs", vsphere);
            if (!new File(this.storageConfig.getTelegrafPath()).exists()) {
                new File(this.storageConfig.getTelegrafPath()).mkdirs();
            }
            tomlWriter.write(inputs, new File(this.storageConfig.getTelegrafPath() + vendor.getUuid() + ".conf"));
        } catch (Exception e) {
            throw e;
        }
    }

    @Override
    public void deleteConfigFile(RequestContext context, CloudVendor vendor) {
        if (vendor != null && CloudProvider.VMWARE.name().equals(vendor.getType())) {
            File file = new File(this.storageConfig.getTelegrafPath() + vendor.getUuid() + ".conf");
            if (file.exists() && !file.delete()) {
                log.error("file delete failed, path:{} " + file.getPath());
            }
            reload();
        }
    }

    @Override
    public GeneralResult reload() {
        String telegrafHost = "";
        int telegrafPort = 22;
        String telegrafUser = "";
        String telegrafPassword = "";
        try {
            String data = this.zooKeeperClient.getData(this.registryClient.getService().getSystem());
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                telegrafHost = object.getString("TelegrafHost");
                telegrafPort = object.getIntValue("TelegrafPort");
                telegrafUser = object.getString("TelegrafUser");
                telegrafPassword = object.getString("TelegrafPassword");
                if (StringUtils.isNotBlank(telegrafPassword)) {
                    // 主机密码解密操作
                    telegrafPassword = new DESEncryptor().decrypt(telegrafPassword, "BocloudCMP");
                }
            }
        } catch (Exception e) {
            log.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(e));
        }

        try {
            SshdExecutor executor = new SshdExecutor(telegrafHost, telegrafPort, telegrafUser, telegrafPassword);
            executor.execute(isDocker ? "sudo docker restart telegraf" : "sudo systemctl restart telegraf.service");
            return new GeneralResult<>(true, "更新采集器配置成功");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }
}
