package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.intf.VmService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.service.adapter.BmsAdapter;
import com.bocloud.cms.service.adapter.CceAdapter;
import com.bocloud.cms.service.adapter.VmAdapter;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.BareMetalInfo;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.entity.ContainerCluster;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.DateTools;
import jakarta.servlet.http.HttpServletRequest;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 云主机Service层
 *
 * @author zhangyf
 * @since 2022/2/21 18:01
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class VmServiceImpl implements VmService {

    private final CmpCloudServerService cmpCloudServerService;
    private final ApplicationContext applicationContext;

    @Override
    public GeneralResult<?> list(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult<?> result = cmpCloudServerService.list(pager, request.getHeader(Common.TOKEN));
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<CloudServerModel> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServerModel.class);
        if (list.isEmpty()) {
            return result;
        }
        // 默认全部开启
        for (CloudServerModel cloudServerModel : list) {
            cloudServerModel.setAlarmEnable(true);
        }
        Map<String, VmAdapter> adapters = applicationContext.getBeansOfType(VmAdapter.class);
        adapters.forEach((s, vmAdapter) -> vmAdapter.list(list, context));
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public GeneralResult vmOverview(Long id) {
        GeneralResult<CloudServer> cloudServer = cmpCloudServerService.detail(id);
        Assert.isTrue(cloudServer.isSuccess(), " 未查询到该云主机相关信息");
        HashMap<Object, Object> result = new HashMap<>(16, 0.75F);
        Date date = cloudServer.getData().getGmtCreate();
        result.put("name", cloudServer.getData().getName());
        result.put("gmtCreate", DateTools.formatTime(date));
        result.put("ip", cloudServer.getData().getManagerIp());
        result.put("powerStatus", cloudServer.getData().getStatus());
        return new GeneralResult<>(true, result, "查询成功");
    }

    public GeneralResult listManagerBms(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult<?> result = cmpCloudServerService.listManagerBms(pager, request.getHeader(Common.TOKEN));
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<BareMetalInfo> list = data.getJSONArray(Common.ROWS).toJavaList(BareMetalInfo.class);
        if (list.isEmpty()) {
            return result;
        }
        // 默认全部开启
        for (BareMetalInfo bareMetalInfo : list) {
            bareMetalInfo.setAlarmEnable(true);
        }
        Map<String, BmsAdapter> adapters = applicationContext.getBeansOfType(BmsAdapter.class);
        adapters.forEach((s, vmAdapter) -> vmAdapter.list(list, context));
        data.put(Common.ROWS, list);
        return result;
    }

    @Override
    public GeneralResult listManagerCce(Pager pager, RequestContext context, HttpServletRequest request) {
        GeneralResult<?> result = cmpCloudServerService.listManagerCce(pager, request.getHeader(Common.TOKEN));
        if (result.isFailed()) {
            return result;
        }
        JSONObject data = (JSONObject) result.getData();
        List<ContainerCluster> list = data.getJSONArray(Common.ROWS).toJavaList(ContainerCluster.class);
        if (list.isEmpty()) {
            return result;
        }
        // 默认全部开启
        for (ContainerCluster containerCluster : list) {
            containerCluster.setAlarmEnable(true);
        }
        Map<String, CceAdapter> adapters = applicationContext.getBeansOfType(CceAdapter.class);
        adapters.forEach((s, vmAdapter) -> vmAdapter.list(list, context));
        data.put(Common.ROWS, list);
        return result;
    }

}
