package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.CmpServerService;
import com.bocloud.ims.entity.Server;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

/**
 * Vmware宿主机
 *
 * @author zhangyf
 * @since 2023/05/26
 */
@Service
@Slf4j
public class VmwareHostServiceImpl implements MonitoringAble<Server>, InitializingBean {

    @Autowired
    private CmpServerService cmpServerService;
    @Autowired
    private ResourceAlarmStatusService resourceAlarmStatusService;

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_VCENTER_HOST, this);
    }

    @Override
    public List<Server> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);
        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpServerService.list(new Pager(1, Integer.MAX_VALUE, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(Server.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(Server targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        if ("vsphere_host_success".equals(metric.getExpr())) {
            params.put("source", targetObject.getManageIp());
        } else {
            params.put("source", targetObject.getName());
        }
        return params;
    }

    @Override
    public Server parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String name = String.valueOf(alert.getLabels().get("source"));
        List<Param> params = Collections.singletonList((new Param(MapTools.simpleMap("name", name), Sign.EQ)));
        GeneralResult result = cmpServerService.list(new Pager(1, 1, params, null));
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<Server> list = data.getJSONArray(Common.ROWS).toJavaList(Server.class);
        Assert.isTrue(list != null && !list.isEmpty(), "not found server,name:" + name);
        return list.get(0);
    }

    @Override
    public void destroy(Server object, String catalog) throws Exception {

    }

    @Override
    public void wrapAlarm(Alarm alarm, Server object) {
        if (object.getName() != null & object.getManageIp() != null) {
            alarm.setTargetName(object.getName() + "(" + object.getManageIp() + ")");
        } else {
            String targetName = object.getManageIp() == null ? object.getName() : object.getManageIp();
            alarm.setTargetName(targetName);
        }
        alarm.setVendorId(object.getVendorId());
        alarm.setOwnerId(object.getCreatorId());
        alarm.setTarget(String.valueOf(object.getId()));
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, Server object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}
