package com.bocloud.cms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.Alarm;
import com.bocloud.cms.entity.Metric;
import com.bocloud.cms.entity.Rule;
import com.bocloud.cms.entity.RuleGroup;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.AlarmService;
import com.bocloud.cms.intf.MonitoringAble;
import com.bocloud.cms.intf.ResourceAlarmStatusService;
import com.bocloud.cms.model.PrometheusAlarm;
import com.bocloud.cms.service.internal.CmpCloudServerService;
import com.bocloud.ims.entity.CloudServer;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.util.*;

/**
 * Vmware云主机
 *
 * @author zhangyf
 * @since 2021/10/14
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class VmwareVmServiceImpl implements MonitoringAble<CloudServer>, InitializingBean {

    private final CmpCloudServerService cmpCloudServerService;
    private final ResourceAlarmStatusService resourceAlarmStatusService;
    private final AlarmService alarmService;

    @Override
    public void afterPropertiesSet() throws Exception {
        MonitoringAbleRegistry.register(MonitorResourceType.MONITOR_VCENTER_VM, this);
    }

    @Override
    public List<CloudServer> listTargetObjects(List<String> targetIds, MonitorResourceType resourceType) {
        List<Integer> targetList = new ArrayList<>();
        Optional.ofNullable(targetIds).orElse(new ArrayList<>()).forEach(targetId -> targetList.add(Integer.valueOf(targetId)));
        Param param = new Param(MapTools.simpleMap("id", JSONObject.toJSONString(targetList).substring(1, JSONObject.toJSONString(targetList).length() - 1)), Sign.IN);

        List<Param> params = Collections.singletonList(param);
        GeneralResult result = cmpCloudServerService.list(new Pager(1, Integer.MAX_VALUE, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        return data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
    }

    @Override
    public Map<String, String> getPrometheusExprParam(CloudServer targetObject, Metric metric) {
        Map<String, String> params = new HashMap<>(16, 0.75F);
        params.put("uuid", targetObject.getBiosUuid());
        return params;
    }

    @Override
    public CloudServer parseObjectFromAlert(PrometheusAlarm.PrometheusAlert alert, RuleGroup ruleGroup, Rule rule, Metric metric) {
        String uuid = String.valueOf(alert.getLabels().get("uuid"));
        List<Param> params = new ArrayList<>(1);
        params.add(new Param(MapTools.simpleMap("biosUuid", uuid), Sign.EQ));
        GeneralResult result = cmpCloudServerService.list(new Pager(1, 1, params, null), null);
        Assert.isTrue(result.isSuccess(), result.getMessage());
        JSONObject data = (JSONObject) result.getData();
        List<CloudServer> list = data.getJSONArray(Common.ROWS).toJavaList(CloudServer.class);
        Assert.isTrue(list != null && !list.isEmpty(), "not found vm,biosUuid:" + uuid);
        return list.get(0);
    }

    @Override
    public void destroy(CloudServer cloudServer, String catalog) throws Exception {
        log.info("destroy vm : " + cloudServer.getId());
        alarmService.solveByCloudServer(cloudServer.getId(), catalog, 1L, "系统解决");
    }

    @Override
    public void wrapAlarm(Alarm alarm, CloudServer vm) {
        alarm.setTarget(vm.getId().toString());
        if (vm.getName() != null & vm.getManagerIp() != null) {
            alarm.setTargetName(vm.getName() + "(" + vm.getManagerIp() + ")");
        } else {
            String targetName = vm.getManagerIp() == null ? vm.getName() : vm.getManagerIp();
            alarm.setTargetName(targetName);
        }
        alarm.setOsCategory(vm.getOsCategory());
        alarm.setVendorId(vm.getVendorId());
        alarm.setOwnerId(vm.getCreatorId());
    }

    @Override
    public boolean alarmEnable(RuleGroup ruleGroup, CloudServer object) {
        return resourceAlarmStatusService.alarmEnable(ruleGroup.getResourceType(), String.valueOf(object.getId()), ruleGroup.getTenantId());
    }
}
