package com.bocloud.cms.service.adapter;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * H3C
 *
 * @author zhangyf
 */
@Slf4j
@Component
public class CloudOsAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public CloudOsAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.CLOUDOS.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        String cpuExpr = "round(cloudOs_vm_cpu_util{instanceId=~'" + instanceId + "'},0.01)";
        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr);
        String memExpr = "round(cloudOs_vm_memory_util{instanceId=~'" + instanceId + "'},0.01)";
        List<PrometheusResult> memoryList = prometheusService.query(memExpr);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_CLOUDOS_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instanceId").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }

    }


}
