package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 华为云桌面
 *
 * @author wpj
 * @since 2022/8/18
 */
@Component
public class FusionComputerAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository alarmStatusRepository;

    public FusionComputerAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository alarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.alarmStatusRepository = alarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.HWACCESS.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<PrometheusResult> cpuList = prometheusService.query("round(fusioncompute_vm_cpu_usage{urn=~'" + instanceId + "'},0.01)");
        List<PrometheusResult> memoryList = prometheusService.query("round(fusioncompute_vm_mem_usage{urn=~'" + instanceId + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.alarmStatusRepository.list(MonitorResourceType.MONITOR_HWACCESS_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            //CPU利用率
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("urn").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                    break;
                }
            }
            //内存利用率
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("urn").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                    break;
                }
            }
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }
}