package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 金山
 *
 * @author zhangyf
 */
@Component
public class KsyunAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public KsyunAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.KINGCLOUD.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> params = Collections.singletonList(new LabelParam("instance_id", instanceId, Operator.RE_EQ));

        String cpuExpr = "round(ksyun_vm_cpu_utilizition_total{}, 0.01)";
        String memoryExpr = "round(ksyun_vm_memory_utilizition_total{}, 0.01)";

        List<PrometheusResult> cpuList = prometheusService.query(cpuExpr, params);
        List<PrometheusResult> memoryList = prometheusService.query(memoryExpr, params);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_KINGCLOUD_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            for (PrometheusResult cpuUsage : cpuList) {
                if (cpuUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memoryUsage : memoryList) {
                if (memoryUsage.getMetric().get("instance_id").equals(cloudServer.getInstanceId())) {
                    cloudServer.setMemUsage(String.valueOf(memoryUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }

    }

}
