package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.enums.Operator;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.LabelParam;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * OpenStack
 *
 * @author zhangyf
 */
@Component
public class OpenStackAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public OpenStackAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> instanceIds = list.stream()
                .filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.OPENSTACK.name()))
                .map(CloudServer::getInstanceId).collect(Collectors.toList());
        if (ListTool.isEmpty(instanceIds)) {
            return;
        }
        String instanceId = StringUtils.join(instanceIds.toArray(), "|");
        List<LabelParam> labelParams = Arrays.asList(new LabelParam("instanceId", instanceId, Operator.RE_EQ));
        //拼接参数
        String memoryExpr = "round((1-libvirt_domain_memory_stat_unused/libvirt_domain_memory_stat_available{})*100,0.01)";
        String cpuExpr = "round((avg(irate(libvirt_domain_info_cpu_time_seconds_total{}[5m])) by (instanceId))*100/ avg(libvirt_domain_info_virtual_cpus) by (instanceId),0.01)";
        List<PrometheusResult> memory = prometheusService.query(memoryExpr, labelParams);
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr, labelParams);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_OPENSTACK_VM.name(), target, context);

        for (CloudServerModel cloudServer : list) {
            cloudServer.setMemTotal(String.valueOf(cloudServer.getMemory()));
            //CPU、内存利用率
            Optional.ofNullable(cpu).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("instanceId").equals(cloudServer.getInstanceId()))
                    .findFirst().ifPresent(s -> cloudServer.setCpuUsage(String.valueOf(s.getValue()[1])));
            Optional.ofNullable(memory).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("instanceId").equals(cloudServer.getInstanceId()))
                    .findFirst().ifPresent(s -> cloudServer.setMemUsage(String.valueOf(s.getValue()[1])));

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }

    }

}
