package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.model.ServerModel;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Vmware
 *
 * @author zhangyf
 */
@RequiredArgsConstructor
@Component
public class OpenStackHostAdapter implements HostAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    @Override
    public void list(List<ServerModel> list, RequestContext context) {
        List<String> vmIds = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.OPENSTACK.name()))
                .map(s -> String.valueOf(s.getId())).collect(Collectors.toList());
        if (ListTool.isEmpty(vmIds)) {
            return;
        }
        String cpuExpr = "round((1 - avg(irate(node_cpu_seconds_total{host_id=~'" + StringUtils.join(vmIds.toArray(), "|") + "',mode='idle'}[5m])) by (host_id))*100,0.01)";
        String memoryExpr = "round(100 - ((node_memory_MemFree_bytes+node_memory_Cached_bytes+node_memory_Buffers_bytes)/node_memory_MemTotal_bytes{host_id=~'" + StringUtils.join(vmIds.toArray(), "|") + "'}) * 100,0.01)";
        List<PrometheusResult> cpu = prometheusService.query(cpuExpr);
        List<PrometheusResult> memory = prometheusService.query(memoryExpr);

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_OPENSTACK_HOST.name(), target, context);

        for (ServerModel server : list) {
            for (PrometheusResult cpuUsage : cpu) {
                if (cpuUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setCpuUsage(String.valueOf(cpuUsage.getValue()[1]));
                }
            }
            for (PrometheusResult memUsage : memory) {
                if (memUsage.getMetric().get("host_id").equals(String.valueOf(server.getId()))) {
                    server.setMemUsage(String.valueOf(memUsage.getValue()[1]));
                }
            }
            //告警启用状态，默认开启
            boolean alarmEnable = Optional.ofNullable(alarmStatuses).orElse(new ArrayList<>()).stream()
                    .noneMatch(a -> a.getResourceId().equals(server.getId().toString()));
            server.setAlarmEnable(alarmEnable);
        }
    }
}
