package com.bocloud.cms.service.adapter;

import com.bocloud.cms.entity.ResourceAlarmStatus;
import com.bocloud.cms.enums.MonitorResourceType;
import com.bocloud.cms.intf.PrometheusService;
import com.bocloud.cms.model.CloudServerModel;
import com.bocloud.cms.model.PrometheusResult;
import com.bocloud.cms.repository.ResourceAlarmStatusRepository;
import com.bocloud.ims.entity.CloudServer;
import com.bocloud.ims.enums.CloudProvider;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Vmware
 *
 * @author zhangyf
 */
@Component
public class VmwareAdapter implements VmAdapter {

    private final PrometheusService prometheusService;
    private final ResourceAlarmStatusRepository resourceAlarmStatusRepository;

    public VmwareAdapter(PrometheusService prometheusService, ResourceAlarmStatusRepository resourceAlarmStatusRepository) {
        this.prometheusService = prometheusService;
        this.resourceAlarmStatusRepository = resourceAlarmStatusRepository;
    }

    @Override
    public void list(List<CloudServerModel> list, RequestContext context) {
        List<String> vmnames = list.stream().filter(s -> s.getVendorType().equalsIgnoreCase(CloudProvider.VMWARE.name()))
                .map(CloudServer::getName).collect(Collectors.toList());
        if (ListTool.isEmpty(vmnames)) {
            return;
        }
        String vmname = StringUtils.join(vmnames.toArray(), "|");
        vmname = vmname.replaceAll("\\(", "\\\\\\\\(");
        vmname = vmname.replaceAll("\\)", "\\\\\\\\)");
        List<PrometheusResult> memory = prometheusService.query("round(vsphere_vm_mem_usage_average{vmname=~'" + vmname + "'},0.01)");
        List<PrometheusResult> cpu = prometheusService.query("round(vsphere_vm_cpu_usage_average{vmname=~'" + vmname + "'},0.01)");

        List<String> target = list.stream().map(m -> String.valueOf(m.getId())).collect(Collectors.toList());
        List<ResourceAlarmStatus> alarmStatuses = this.resourceAlarmStatusRepository.list(MonitorResourceType.MONITOR_VCENTER_VM.name(), target, context);
        for (CloudServerModel cloudServer : list) {
            cloudServer.setMemTotal(String.valueOf(cloudServer.getMemory()));
            //CPU、内存利用率
            Optional.ofNullable(cpu).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("vmname").equals(cloudServer.getName()))
                    .findFirst().ifPresent(s -> cloudServer.setCpuUsage(String.valueOf(s.getValue()[1])));
            Optional.ofNullable(memory).orElse(new ArrayList<>()).stream()
                    .filter(s -> s.getMetric().get("vmname").equals(cloudServer.getName()))
                    .findFirst().ifPresent(s -> cloudServer.setMemUsage(String.valueOf(s.getValue()[1])));

            //告警启用状态，默认开启
            if (alarmStatuses.size() > 0) {
                for (ResourceAlarmStatus alarmStatus : alarmStatuses) {
                    if (alarmStatus.getResourceId().equals(cloudServer.getId().toString())) {
                        cloudServer.setAlarmEnable(alarmStatus.getEnable());
                    }
                }
            }
        }
    }


}
