package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.ims.entity.Server;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 宿主机
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class CmpServerService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    public CmpServerService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 宿主机详情
     *
     * @param id 宿主机ID
     * @return 宿主机详情
     */
    public GeneralResult<Server> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/hosts/" + id,
                HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, Server.class), "success");
    }

    /**
     * 宿主机列表
     *
     * @param pager 分页
     * @return 宿主机列表
     */
    public GeneralResult<?> list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()), "/v1/hosts",
                HttpMethod.GET, headers, pager.toMap());
        return remoteService.invoke();
    }

}
