package com.bocloud.cms.service.internal;

import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * HCSO内部调用
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class HcsoService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    private static final Service SERVICE = Service.create(BoCloudService.CMP.getName());

    public HcsoService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * Nat网关列表
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return Nat网关列表
     */
    public GeneralResult<?> listOfNat(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/natgateway", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }


    /**
     * 磁盘列表查询
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return 磁盘列表
     */
    public GeneralResult<?> listOfVolume(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/volumes", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * RDS列表查询
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return RDS列表
     */
    public GeneralResult<?> listOfRds(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/cloudrds", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 负载均衡列表查询
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return 负载均衡列表
     */
    public GeneralResult<?> listOfLoadBalancer(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/lbs", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 负载均衡监听器列表查询
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return 负载均衡监听器列表
     */
    public GeneralResult<?> listOfLbListener(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/lb/listeners", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 华为云公网IP  弹性EIP列表查询
     *
     * @param pager 分页
     * @param token Token,如果为null，则使用internal_token内部调用查询
     * @return 华为云公网IP列表
     */
    public GeneralResult<?> listOfEip(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        if (token == null) {
            headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        } else {
            headers.put(Common.TOKEN, token);
        }
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/hcso/v1/elasticips", HttpMethod.GET,
                headers, pager.toMap(false));
        return remoteService.invoke();
    }

}
