package com.bocloud.cms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.sms.entity.Tenant;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * @author fanzx
 * @date 2021/09/18
 */
@Component
public class SmsTenantService {

    private static final Service SERVICE = Service.create(BoCloudService.SMS.getName());

    private final CurrentService currentService;
    private final ServiceFactory serviceFactory;

    public SmsTenantService(CurrentService currentService, ServiceFactory serviceFactory) {
        this.currentService = currentService;
        this.serviceFactory = serviceFactory;
    }

    /**
     * 获取租户详情
     *
     * @param id 租户id
     * @return 租户信息
     */
    public GeneralResult<Tenant> detail(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/tenants/" + id, HttpMethod.GET, headers, null);
        GeneralResult result = remoteService.invoke();
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(result.isSuccess(), JSONObject.parseObject(data, Tenant.class), result.getMessage());
    }

}
