package com.bocloud.cms.service.internal.kubernetes;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.ims.entity.kubernetes.KubeNode;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * Kubernetes
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@RequiredArgsConstructor
@Component
public class KubernetesInternalService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    /**
     * 分页查询节点列表
     *
     * @param pager 分页
     * @return 节点列表
     */
    public GeneralResult<?> listOfNode(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()),
                "/plugins/kubernetes/v1/nodes", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 分页查询Pod列表
     *
     * @param pager 分页
     * @return POD列表
     */
    public GeneralResult<?> listOfPod(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()),
                "/plugins/kubernetes/v1/pods", HttpMethod.GET, headers, pager.toMap(false));
        return remoteService.invoke();
    }

    /**
     * 节点详情
     *
     * @param id 节点ID
     * @return 节点
     */
    public GeneralResult<KubeNode> detailOfNode(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.CMP.getName()),
                "/plugins/kubernetes/v1/nodes/" + id, HttpMethod.GET, headers, null);
        GeneralResult<?> result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, KubeNode.class), "success");
    }

}
