package com.bocloud.cms.service.internal.mms;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.pms.entity.Cms;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * 消息队列
 *
 * @author zhangyf
 * @version 5.6.0
 * @date 2022/2/21 18:01
 */
@Component
public class MmsCmsService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;
    private static final Service SERVICE = Service.create(BoCloudService.CMP.getName());

    public MmsCmsService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    /**
     * 详情
     *
     * @param id ID
     * @return 详情
     */
    public Cms detail(Long id) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(SERVICE, "/plugins/mms/v1/cms/" + id + "/basics", HttpMethod.GET, headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            throw new RuntimeException(result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return JSONObject.parseObject(data, Cms.class);
    }

    /**
     * 消息队列列表
     *
     * @param pager 分页
     * @param token Token
     * @return 消息队列列表
     */
    public GeneralResult<?> list(Pager pager, String token) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(Common.TOKEN, token);
        return serviceFactory.build(SERVICE, "/plugins/mms/v1/cms", HttpMethod.GET, headers, pager.toMap()).invoke();
    }

    /**
     * 消息队列列表
     *
     * @param pager 分页
     * @return 消息队列列表
     */
    public GeneralResult list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(2);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        return serviceFactory.build(SERVICE, "/plugins/mms/v1/cms", HttpMethod.GET, headers, pager.toMap()).invoke();
    }

}
